# Copyright 2015 Cisco Systems, Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from __future__ import print_function

import argparse

from networking_cisco._i18n import _

from neutronclient.common import extension
from neutronclient.neutron import v2_0 as neutronV20


ROUTER_TYPE = 'routertype'


def _add_updatable_args(parser):
    parser.add_argument(
        '--name',
        help=_('Name of this router type.'))
    parser.add_argument(
        '--description',
        help=_('Description of this router type.'))
    parser.add_argument(
        '--ha-enabled',
        dest='ha_enabled_by_default',
        action='store_true',
        help=_('Make HA enabled for the router type.'),
        default=argparse.SUPPRESS)
    parser.add_argument(
        '--ha_enabled',
        dest='ha_enabled_by_default',
        action='store_true',
        help=argparse.SUPPRESS,
        default=argparse.SUPPRESS)
    parser.add_argument(
        '--unshared',
        dest='shared',
        action='store_false',
        help=_('Make router type NOT shared among tenants.'),
        default=argparse.SUPPRESS)
    parser.add_argument(
        '--slot-need',
        help=_('Number of slots routers of this type consumes.'))
    parser.add_argument(
        '--slot_need',
        help=argparse.SUPPRESS)


def _updatable_args2body(parsed_args, body):
    neutronV20.update_dict(parsed_args, body[ROUTER_TYPE],
                           ['name', 'description', 'ha_enabled_by_default',
                            'shared', 'slot_need'])


class RouterType(extension.NeutronClientExtension):
    resource = ROUTER_TYPE
    resource_plural = '%ss' % resource
    object_path = '/%s' % resource_plural
    resource_path = '/%s/%%s' % resource_plural
    versions = ['2.0']
    allow_names = True


class RouterTypeList(extension.ClientExtensionList, RouterType):
    """List router types that belong to a given tenant."""

    shell_command = 'cisco-router-type-list'
    list_columns = ['id', 'name', 'description', 'template_id']
    pagination_support = True
    sorting_support = True


class RouterTypeShow(extension.ClientExtensionShow, RouterType):
    """Show information of a given router type."""

    shell_command = 'cisco-router-type-show'


class RouterTypeCreate(extension.ClientExtensionCreate, RouterType):
    """Create a router type for a given tenant."""

    shell_command = 'cisco-router-type-create'

    def add_known_arguments(self, parser):
        _add_updatable_args(parser)
        parser.add_argument(
            '--id',
            help=_('Id for this router type.'))

        parser.add_argument(
            'template_id', metavar='TEMPLATE',
            help=_('Hosting device template to associate router type with.'))
        parser.add_argument(
            'scheduler',
            metavar='SCHEDULER',
            help=_('Scheduler module to use for routers of this router type.'))
        parser.add_argument(
            'driver',
            metavar='PLUGIN_DRIVER',
            help=_('Driver module to use in plugin for routers of this router '
                   'type.'))
        parser.add_argument(
            'cfg_agent_service_helper',
            metavar='SERVICE_HELPER',
            help=_('Service helper in config agent to use for routers of this '
                   'router type.'))
        parser.add_argument(
            'cfg_agent_driver',
            metavar='AGENT_DRIVER',
            help=_('Device driver in config agent to use for routers of this '
                   'router type.'))

    def args2body(self, parsed_args):
        client = self.get_client()
        _template_id = neutronV20.find_resourceid_by_name_or_id(
            client, 'hosting_device_template', parsed_args.template_id)
        body = {self.resource: {
            'template_id': _template_id,
            'scheduler': parsed_args.scheduler,
            'driver': parsed_args.driver,
            'cfg_agent_service_helper': parsed_args.cfg_agent_service_helper,
            'cfg_agent_driver': parsed_args.cfg_agent_driver}}
        _updatable_args2body(parsed_args, body)
        neutronV20.update_dict(parsed_args, body[self.resource],
                               ['id', 'tenant_id'])
        return body


class RouterTypeDelete(extension.ClientExtensionDelete, RouterType):
    """Delete a given router type."""

    shell_command = 'cisco-router-type-delete'


class RouterTypeUpdate(extension.ClientExtensionUpdate, RouterType):
    """Update router type's information."""

    shell_command = 'cisco-router-type-update'

    def add_known_arguments(self, parser):
        # adding ha_disabled here so that it is available for update only as
        # HA is disabled by default and not meaningful in the create operation
        parser.add_argument(
            '--ha-disabled',
            dest='ha_enabled_by_default',
            action='store_false',
            help=_('Make HA disabled for the router type.'),
            default=argparse.SUPPRESS)
        parser.add_argument(
            '--ha_disabled',
            dest='ha_enabled_by_default',
            action='store_false',
            help=argparse.SUPPRESS,
            default=argparse.SUPPRESS)
        # adding shared here so that it is available for update only as it is
        # True by default and not meaningful in the create operation
        parser.add_argument(
            '--shared',
            dest='shared',
            action='store_true',
            help=_('Make router type shared among tenants.'),
            default=argparse.SUPPRESS)
        _add_updatable_args(parser)

    def args2body(self, parsed_args):
        body = {self.resource: {}}
        _updatable_args2body(parsed_args, body)
        return body



# Copyright 2014 Cisco Systems, Inc.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import sqlalchemy as sa
from sqlalchemy import orm

from networking_cisco import backwards_compatibility as bc
from networking_cisco.plugins.cisco.db.device_manager import hd_models


class RouterType(bc.model_base.BASEV2, bc.model_base.HasId, bc.HasProject):
    """Represents Neutron router types.

    A router type is associated with a with hosting device template.
    The template is used when hosting device for the router type is created.

    Only 'id', 'name', 'description' are visible in non-admin context.
    """
    __tablename__ = 'cisco_router_types'

    # name of router type, should preferably be unique
    name = sa.Column(sa.String(255), nullable=False)
    # description of this router type
    description = sa.Column(sa.String(255))
    # template to use to create hosting devices for this router type
    template_id = sa.Column(sa.String(36),
                            sa.ForeignKey('cisco_hosting_device_templates.id',
                                          ondelete='CASCADE'))
    template = orm.relationship(hd_models.HostingDeviceTemplate)
    # 'ha_enabled_by_default' is True if routers of this type should have HA on
    ha_enabled_by_default = sa.Column(sa.Boolean, nullable=False,
                                      server_default=sa.sql.false())
    # 'shared' is True if routertype is available to all tenants
    shared = sa.Column(sa.Boolean, nullable=False,
                       server_default=sa.sql.true())
    #TODO(bobmel): add HA attribute: One of None, 'GPLB', 'VRRP', or 'HSRP'
    # The number of slots this router type consume in hosting device
    slot_need = sa.Column(sa.Integer, autoincrement=False)
    # module to be used as scheduler for router of this type
    scheduler = sa.Column(sa.String(255), nullable=False)
    # module to be used by router plugin as router type driver
    driver = sa.Column(sa.String(255), nullable=False)
    # module to be used by configuration agent as service helper driver
    cfg_agent_service_helper = sa.Column(sa.String(255), nullable=False)
    # module to be used by configuration agent for in-device configurations
    cfg_agent_driver = sa.Column(sa.String(255), nullable=False)


class RouterHostingDeviceBinding(bc.model_base.BASEV2):
    """Represents binding between Neutron routers and their hosting devices."""
    __tablename__ = 'cisco_router_mappings'

    router_id = sa.Column(sa.String(36),
                          sa.ForeignKey('routers.id', ondelete='CASCADE'),
                          primary_key=True)
    router = orm.relationship(
        bc.Router,
        backref=orm.backref('hosting_info', cascade='all', uselist=False))
    # 'router_role' specifies the type of role the router serves in
    role = sa.Column(sa.String(255), default=None)
    # 'router_type_id' is id of router type for this router
    router_type_id = sa.Column(
        sa.String(36),
        sa.ForeignKey('cisco_router_types.id'),
        primary_key=True,
        nullable=False)
    router_type = orm.relationship(RouterType)
    # 'inflated_slot_need' is the slot need of the router plus the
    # number slots needed by other resources to be associated with the
    # router. It's only considered if > 0.
    inflated_slot_need = sa.Column(sa.Integer, autoincrement=False,
                                   server_default='0')
    # If 'auto_schedule' is True then router is automatically scheduled
    # if it lacks a hosting device or its hosting device fails.
    auto_schedule = sa.Column(sa.Boolean, default=True, nullable=False)
    share_hosting_device = sa.Column(sa.Boolean, nullable=False,
                                     server_default=sa.sql.true())
    # id of hosting device hosting this router, None/NULL if unscheduled.
    hosting_device_id = sa.Column(sa.String(36),
                                  sa.ForeignKey('cisco_hosting_devices.id',
                                                ondelete='SET NULL'))
    hosting_device = orm.relationship(hd_models.HostingDevice)



# Copyright 2016 Cisco Systems, Inc.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os

import mock
from neutron.tests import base
from oslo_config import cfg
from oslo_serialization import jsonutils

from networking_cisco.plugins.cisco.cfg_agent.device_drivers.asr1k import (
    asr1k_cfg_syncer)
from networking_cisco.plugins.cisco.cfg_agent.device_drivers.asr1k import (
    asr1k_routing_driver as driver)
from networking_cisco.plugins.cisco.common.htparser import HTParser


cfg.CONF.register_opts(driver.ASR1K_DRIVER_OPTS, "multi_region")

ASR_RUNNING_CFG_NO_R2_JSON_JUST_INTERFACES = [
    "interface Port-channel10",
    " no ip address",
    "!",
    "interface Port-channel10.165",
    " description OPENSTACK_NEUTRON_0000001_INTF",
    " encapsulation dot1Q 165",
    " ip address 10.23.229.153 255.255.255.240",
    " ip nat outside",
    " standby delay minimum 30 reload 60",
    " standby version 2",
    " standby 1064 ip 10.23.229.152",
    " standby 1064 timers 1 3",
    " standby 1064 priority 20",
    " standby 1064 name neutron-hsrp-1064-165",
    "!",
    "interface Port-channel10.2005",
    " description OPENSTACK_NEUTRON_0000001_INTF",
    " encapsulation dot1Q 2005",
    " vrf forwarding nrouter-9ad979-0000001",
    " ip address 192.168.3.15 255.255.255.0",
    " ip nat inside",
    " standby delay minimum 30 reload 60",
    " standby version 2",
    " standby 1064 ip 192.168.3.1",
    " standby 1064 timers 1 3",
    " standby 1064 priority 10",
    " standby 1064 name neutron-hsrp-1064-2005",
    "!",
    "interface Port-channel10.2029",
    " description OPENSTACK_NEUTRON_0000001_INTF",
    " encapsulation dot1Q 2029",
    " vrf forwarding nrouter-9ad979-0000001",
    " ip address 192.168.2.15 255.255.255.0",
    " ip nat inside",
    " standby delay minimum 30 reload 60",
    " standby version 2",
    " standby 1064 ip 192.168.2.1",
    " standby 1064 timers 1 3",
    " standby 1064 priority 10",
    " standby 1064 name neutron-hsrp-1064-2029",
    "!",
    "interface Port-channel10.2031",
    " description OPENSTACK_NEUTRON_0000001_INTF",
    " encapsulation dot1Q 2031",
    " vrf forwarding nrouter-9ad979-0000001",
    " ip address 192.168.1.30 255.255.255.0",
    " ip nat inside",
    " standby delay minimum 30 reload 60",
    " standby version 2",
    " standby 1064 ip 192.168.1.1",
    " standby 1064 timers 1 3",
    " standby 1064 priority 10",
    " standby 1064 name neutron-hsrp-1064-2031",
    "!",
    "interface Port-channel10.2044",
    " description OPENSTACK_NEUTRON_0000001_INTF",
    " encapsulation dot1Q 2044",
    " vrf forwarding nrouter-9ad979-0000001",
    " ip address 192.168.0.4 255.255.255.0",
    " ip nat inside",
    " standby delay minimum 30 reload 60",
    " standby version 2",
    " standby 1064 ip 192.168.0.1",
    " standby 1064 timers 1 3",
    " standby 1064 priority 10",
    " standby 1064 name neutron-hsrp-1064-2044",
    "!",
    "interface Port-channel10.2499",
    " ip nat outside",
    "!",
    "interface TenGigabitEthernet0/0/0",
    " no ip address",
    " cdp enable",
    " channel-group 10 mode active",
    "!",
    "interface TenGigabitEthernet0/1/0",
    " no ip address",
    " cdp enable",
    " channel-group 10 mode active",
    "!",
    "interface TenGigabitEthernet0/3/0",
    " no ip address",
    " shutdown",
    "!",
    "interface GigabitEthernet0",
    " vrf forwarding Mgmt-intf",
    " ip address 172.20.231.19 255.255.255.0",
    " negotiation auto",
    " no mop enabled",
    "!"
]

# A basic ASR running-cfg that doesn't have multi-region enabled
ASR_BASIC_RUNNING_CFG_NO_MULTI_REGION = [
    "!",
    "vrf definition Mgmt-intf",
    " !",
    " address-family ipv4",
    " exit-address-family",
    " !",
    " address-family ipv6",
    " exit-address-family",
    "!",
    "vrf definition nrouter-3ea5f9",
    " !",
    " address-family ipv4",
    " exit-address-family",
    " !",
    " address-family ipv6",
    " exit-address-family",
    "!",
    "interface Port-channel10",
    " no ip address",
    "!",
    "interface Port-channel10.2564",
    " description OPENSTACK_NEUTRON_INTF",
    " encapsulation dot1Q 2564",
    " vrf forwarding nrouter-3ea5f9",
    " ip address 10.2.0.4 255.255.255.0",
    " ip nat inside",
    " standby delay minimum 30 reload 60",
    " standby version 2",
    " standby 1064 ip 10.2.0.1",
    " standby 1064 timers 1 3",
    " standby 1064 priority 97",
    " standby 1064 name neutron-hsrp-1064-2564",
    "!",
    "interface Port-channel10.3000",
    " description OPENSTACK_NEUTRON_INTF",
    " encapsulation dot1Q 3000",
    " ip address 172.16.0.105 255.255.0.0",
    " ip nat outside",
    " standby delay minimum 30 reload 60",
    " standby version 2",
    " standby 1064 ip 172.16.0.104",
    " standby 1064 timers 1 3",
    " standby 1064 priority 97",
    " standby 1064 name neutron-hsrp-1064-3000",
    "!",
    "interface TenGigabitEthernet0/0/0",
    " no ip address",
    " cdp enable",
    " channel-group 10 mode active",
    "!",
    "interface TenGigabitEthernet0/1/0",
    " no ip address",
    " cdp enable",
    " channel-group 10 mode active",
    "!",
    "interface TenGigabitEthernet0/3/0",
    " no ip address",
    " shutdown",
    "!",
    "interface GigabitEthernet0",
    " vrf forwarding Mgmt-intf",
    " ip address 172.20.231.19 255.255.255.0",
    " negotiation auto",
    " no mop enabled",
    "!",
    "ip nat pool nrouter-3ea5f9_nat_pool 172.16.0.124"
    " 172.16.0.124 netmask 255.255.0.0",
    "ip nat inside source static 10.2.0.5 172.16.0.126"
    " vrf nrouter-3ea5f9 redundancy neutron-hsrp-1064-3000",
    "ip nat inside source list neutron_acl_2564_47f1a63e pool"
    " nrouter-3ea5f9_nat_pool vrf nrouter-3ea5f9 overload",
    "ip forward-protocol nd",
    "!",
    "ip route vrf Mgmt-intf 0.0.0.0 0.0.0.0 172.20.231.1",
    "ip route vrf nrouter-3ea5f9 0.0.0.0 0.0.0.0"
    " Port-channel10.3000 172.16.0.1",
    "!",
    "ip access-list standard neutron_acl_2564_47f1a63e",
    " permit 10.2.0.0 0.0.0.255",
    "!",
    "end",
    ""
]

# basic ASR running cfg emulating an openstack deployment with
# region label 0000002
ASR_BASIC_RUNNING_CFG = [
    "!",
    "vrf definition Mgmt-intf",
    " !",
    " address-family ipv4",
    " exit-address-family",
    " !",
    " address-family ipv6",
    " exit-address-family",
    "!",
    "vrf definition nrouter-3ea5f9-0000002",
    " !",
    " address-family ipv4",
    " exit-address-family",
    " !",
    " address-family ipv6",
    " exit-address-family",
    "!",
    "interface Port-channel10",
    " no ip address",
    "!",
    "interface Port-channel10.2564",
    " description OPENSTACK_NEUTRON_0000002_INTF",
    " encapsulation dot1Q 2564",
    " vrf forwarding nrouter-3ea5f9-0000002",
    " ip address 10.2.0.4 255.255.255.0",
    " ip nat inside",
    " standby delay minimum 30 reload 60",
    " standby version 2",
    " standby 1064 ip 10.2.0.1",
    " standby 1064 timers 1 3",
    " standby 1064 priority 97",
    " standby 1064 name neutron-hsrp-1064-2564",
    "!",
    "interface Port-channel10.3000",
    " description OPENSTACK_NEUTRON_0000002_INTF",
    " encapsulation dot1Q 3000",
    " ip address 172.16.0.105 255.255.0.0",
    " ip nat outside",
    " standby delay minimum 30 reload 60",
    " standby version 2",
    " standby 1064 ip 172.16.0.104",
    " standby 1064 timers 1 3",
    " standby 1064 priority 97",
    " standby 1064 name neutron-hsrp-1064-3000",
    "!",
    "interface TenGigabitEthernet0/0/0",
    " no ip address",
    " cdp enable",
    " channel-group 10 mode active",
    "!",
    "interface TenGigabitEthernet0/1/0",
    " no ip address",
    " cdp enable",
    " channel-group 10 mode active",
    "!",
    "interface TenGigabitEthernet0/3/0",
    " no ip address",
    " shutdown",
    "!",
    "interface GigabitEthernet0",
    " vrf forwarding Mgmt-intf",
    " ip address 172.20.231.19 255.255.255.0",
    " negotiation auto",
    " no mop enabled",
    "!",
    "ip nat pool nrouter-3ea5f9-0000002_nat_pool 172.16.0.124"
    " 172.16.0.124 netmask 255.255.0.0",
    "ip nat inside source static 10.2.0.5 172.16.0.126 vrf"
    " nrouter-3ea5f9-0000002 redundancy neutron-hsrp-1064-3000",
    "ip nat inside source list neutron_acl_0000002_2564_47f1a63e pool"
    " nrouter-3ea5f9-0000002_nat_pool vrf nrouter-3ea5f9-0000002 overload",
    "ip forward-protocol nd",
    "!",
    "ip route vrf Mgmt-intf 0.0.0.0 0.0.0.0 172.20.231.1",
    "ip route vrf nrouter-3ea5f9-0000002 0.0.0.0 0.0.0.0"
    " Port-channel10.3000 172.16.0.1",
    "!",
    "ip access-list standard neutron_acl_0000002_2564_47f1a63e",
    " permit 10.2.0.0 0.0.0.255",
    "!",
    "end",
    ""
]

# with intentionally invalid interfaces configuration that mismatches
# against neutron-db
ASR_RUNNING_CFG_WITH_INVALID_INTFS = [
    "interface Port-channel10",
    " no ip address",
    "!",
    "interface Port-channel10.165",
    " description OPENSTACK_NEUTRON_0000001_INTF",
    " encapsulation dot1Q 165",
    " ip address 10.23.229.153 255.255.255.240",
    " ip nat outside",
    " standby delay minimum 30 reload 60",
    " standby version 2",
    " standby 1064 ip 10.23.229.152",
    " standby 1064 timers 1 3",
    " standby 1064 priority 20",
    " standby 1064 name neutron-hsrp-1064-165",
    "!",
    "interface Port-channel10.2005",
    " description OPENSTACK_NEUTRON_0000001_INTF",
    " encapsulation dot1Q 2005",
    " vrf forwarding nrouter-9ad979-0000001",
    " ip address 192.168.3.15 255.255.255.0",
    " ip nat inside",
    " standby delay minimum 30 reload 60",
    " standby version 2",
    " standby 1064 ip 192.168.3.1",
    " standby 1064 timers 1 3",
    " standby 1064 priority 10",
    " standby 1064 name neutron-hsrp-1064-2005",
    "!",
    "interface Port-channel10.2029",
    " description OPENSTACK_NEUTRON_0000001_INTF",
    " encapsulation dot1Q 2029",
    " vrf forwarding nrouter-9ad979-0000001",
    " ip address 192.168.2.15 255.255.255.0",
    " ip nat inside",
    " standby delay minimum 30 reload 60",
    " standby version 2",
    " standby 1064 ip 192.168.2.1",
    " standby 1064 timers 1 3",
    " standby 1064 priority 10",
    " standby 1064 name neutron-hsrp-1064-2029",
    "!",
    "interface Port-channel10.2031",
    " description OPENSTACK_NEUTRON_0000001_INTF",
    " encapsulation dot1Q 2031",
    " vrf forwarding nrouter-9ad979-0000001",
    " ip address 192.168.1.30 255.255.255.0",
    " ip nat inside",
    " standby delay minimum 30 reload 60",
    " standby version 2",
    " standby 1064 ip 192.168.1.1",
    " standby 1064 timers 1 3",
    " standby 1064 priority 10",
    " standby 1064 name neutron-hsrp-1064-2031",
    "!",
    "interface Port-channel10.2044",
    " description OPENSTACK_NEUTRON_0000001_INTF",
    " encapsulation dot1Q 2044",
    " vrf forwarding nrouter-9ad979-0000001",
    " ip address 192.168.0.4 255.255.255.0",
    " ip nat inside",
    " standby delay minimum 30 reload 60",
    " standby version 2",
    " standby 1064 ip 192.168.0.1",
    " standby 1064 timers 1 3",
    " standby 1064 priority 10",
    " standby 1064 name neutron-hsrp-1064-2044",
    "!",
    "interface Port-channel10.2499",
    " ip nat outside",
    "!",
    "interface Port-channel10.2536",
    " description OPENSTACK_NEUTRON_0000002_INTF",
    " encapsulation dot1Q 2535",
    " vrf forwarding nrouter-92740e-0000002",
    " ip address 192.168.0.7 255.255.255.0",
    " ip nat inside",
    " standby delay minimum 30 reload 60",
    " standby version 2",
    " standby 1064 ip 192.168.0.1",
    " standby 1064 timers 1 3",
    " standby 1064 name neutron-hsrp-1064-2535",
    "!",
    "interface Port-channel10.2564",
    " description OPENSTACK_NEUTRON_0000002_INTF",
    " encapsulation dot1Q 2564",
    " vrf forwarding nrouter-3ea5f9-0000002",
    " ip address 10.2.0.4 255.255.255.0",
    " ip nat inside",
    " standby delay minimum 30 reload 60",
    " standby version 2",
    " standby 1064 ip 10.2.0.1",
    " standby 1064 timers 1 3",
    " standby 1064 priority 97",
    " standby 1064 name neutron-hsrp-1064-2564",
    "!",
    "interface Port-channel10.2577",
    " description OPENSTACK_NEUTRON_0000002_INTF",
    " encapsulation dot1Q 2577",
    " vrf forwarding nrouter-bdc5b5-0000002",
    " ip address 20.20.20.3 255.255.255.0",
    " ip nat inside",
    " standby delay minimum 30 reload 60",
    " standby version 2",
    " standby 1064 ip 20.20.20.1",
    " standby 1064 timers 1 3",
    " standby 1064 name neutron-hsrp-1064-2577",
    "!",
    "interface Port-channel10.3000",
    " description OPENSTACK_NEUTRON_0000002_INTF",
    " encapsulation dot1Q 3000",
    " ip address 172.16.0.105 255.255.0.0",
    " ip nat outside",
    " standby delay minimum 30 reload 60",
    " standby version 2",
    " standby 1064 ip 172.16.0.107",
    " standby 1064 timers 1 3",
    " standby 1064 priority 97",
    " standby 1064 name neutron-hsrp-1064-3000",
    "!",
    "interface TenGigabitEthernet0/0/0",
    " no ip address",
    " cdp enable",
    " channel-group 10 mode active",
    "!",
    "interface TenGigabitEthernet0/1/0",
    " no ip address",
    " cdp enable",
    " channel-group 10 mode active",
    "!",
    "interface TenGigabitEthernet0/3/0",
    " no ip address",
    " shutdown",
    "!",
    "interface GigabitEthernet0",
    " vrf forwarding Mgmt-intf",
    " ip address 172.20.231.19 255.255.255.0",
    " negotiation auto",
    " no mop enabled"
]

TEST_ACL_RUNNING_CFG = [
    " vrf nrouter-bdc5b5-0000002 redundancy neutron-hsrp-1064-3000",
    "ip nat inside source static 10.2.0.5 172.16.0.126"
    " vrf nrouter-3ea5f9-0000002 redundancy neutron-hsrp-1064-3000",
    "ip nat inside source list neutron_acl_0000002_2564_471f1a63e pool"
    " nrouter-3ea5f9-0000002_nat_pool vrf nrouter-3ea5f9-0000002 overload",
    "ip nat inside source list neutron_acl_0000002_2535_86f655d1 pool"
    " nrouter-92740e-0000002_nat_pool vrf nrouter-92740e-0000002 overload",
    "ip nat inside source list neutron_acl_0000001_2005_xxxxxxxx pool"
    " nrouter-9ad979-0000001_nat_pool vrf nrouter-9ad979-0000001 overload",
    "ip nat inside source list neutron_acl_0000001_2029_yyyyyyyy pool"
    " nrouter-9ad979-0000001_nat_pool vrf nrouter-9ad979-0000001 overload",
    "ip nat inside source list neutron_acl_0000001_2031_zzzzzzzz pool"
    " nrouter-9ad979-0000001_nat_pool vrf nrouter-9ad979-0000001 overload",
    "ip nat inside source list neutron_acl_0000001_2044_11111111 pool"
    " nrouter-9ad979-0000001_nat_pool vrf nrouter-9ad979-0000001 overload",
    "ip nat inside source list neutron_acl_0000002_2577_3f70129a pool"
    " nrouter-bdc5b5-0000002_nat_pool vrf nrouter-bdc5b5-0000002 overload",
    "!",
    "ip access-list standard neutron_acl_0000001_2005_xxxxxxxx",
    " permit 192.168.3.0 0.0.0.255",
    "ip access-list standard neutron_acl_0000001_2029_yyyyyyyy",
    " permit 192.168.2.0 0.0.0.255",
    "ip access-list standard neutron_acl_0000001_2031_zzzzzzzz",
    " permit 192.168.1.0 0.0.0.255",
    "ip access-list standard neutron_acl_0000001_2044_11111111",
    " permit 192.168.0.0 0.0.0.255",
    "ip access-list standard neutron_acl_0000002_2535_86f655d1",
    " permit 192.168.0.0 0.0.0.255",
    "ip access-list standard neutron_acl_0000002_2564_47f1a63e",
    " permit 10.2.0.0 0.0.0.255",
    "ip access-list standard neutron_acl_0000002_2577_3f70129a",
    " permit 20.20.20.0 0.0.0.255"
]

# simulated neutron-db dictionary
NEUTRON_DB = [
    {
        "_floatingips": [
            {
                "fixed_ip_address": "20.20.20.7",
                "floating_ip_address": "172.16.0.120",
                "floating_network_id": "7a38a5fd-3ad9-4952-ab08-4def22407269",
                "id": "1b8c01d7-fd60-4e09-9282-9c0085fe8ac6",
                "port_id": "c590b5f8-3fe5-4428-b3c0-fbe82a00d388",
                "router_id": "bdc5b513-7fee-4fe0-9514-78602246d6ee",
                "status": "ACTIVE",
                "tenant_id": "2386c5d814814fb68b037ab7aec784f8"
            },
            {
                "fixed_ip_address": "20.20.20.5",
                "floating_ip_address": "172.16.0.118",
                "floating_network_id": "7a38a5fd-3ad9-4952-ab08-4def22407269",
                "id": "e8f26319-8c42-43ae-92d9-4e1ae789a48d",
                "port_id": "828ac37a-f730-4cc2-bcfc-08e62ca3cdb8",
                "router_id": "bdc5b513-7fee-4fe0-9514-78602246d6ee",
                "status": "ACTIVE",
                "tenant_id": "2386c5d814814fb68b037ab7aec784f8"
            }
        ],
        "_interfaces": [
            {
                "admin_state_up": True,
                "allowed_address_pairs": [],
                "binding:host_id": "asr1k-k7-controller-1-R2",
                "binding:profile": {},
                "binding:vif_details": {
                    "ovs_hybrid_plug": True,
                    "port_filter": True
                },
                "binding:vif_type": "ovs",
                "binding:vnic_type": "normal",
                "device_id": "7bee1420-d86f-43d3-bdd7-38750b77830a",
                "device_owner": "network:router_interface",
                "extra_dhcp_opts": [],
                "extra_subnets": [],
                "fixed_ips": [
                    {
                        "ip_address": "20.20.20.3",
                        "prefixlen": 24,
                        "subnet_id": "90c10f81-47f2-487c-ad1b-c7212c378e08"
                    }
                ],
                "ha_info": {
                    "group": "1064",
                    "ha_port": {
                        "admin_state_up": True,
                        "allowed_address_pairs": [],
                        "binding:host_id": "",
                        "binding:profile": {},
                        "binding:vif_details": {},
                        "binding:vif_type": "unbound",
                        "binding:vnic_type": "normal",
                        "device_id": "bdc5b513-7fee-4fe0-9514-78602246d6ee",
                        "device_owner": "network:router_interface",
                        "extra_dhcp_opts": [],
                        "extra_subnets": [],
                        "fixed_ips": [
                            {
                                "ip_address": "20.20.20.1",
                                "prefixlen": 24,
                                "subnet_id": "90c10f81-47f2-"
                                             "487c-ad1b-c7212c378e08"
                            }
                        ],
                        "id": "eda14976-481b-4dd4-a6c1-835dfd658dbd",
                        "mac_address": "fa:16:3e:1e:bf:be",
                        "name": "",
                        "network_id": "08aedf4f-dd83-4c1c-84f5-759430331eae",
                        "security_groups": [],
                        "status": "DOWN",
                        "subnets": [
                            {
                                "cidr": "20.20.20.0/24",
                                "gateway_ip": "20.20.20.1",
                                "id": "90c10f81-47f2-487c-ad1b-c7212c378e08",
                                "ipv6_ra_mode": None
                            }
                        ],
                        "tenant_id": "2386c5d814814fb68b037ab7aec784f8"
                    },
                    "other_config": "",
                    "timers_config": "",
                    "tracking_config": "",
                    "type": "HSRP"
                },
                "hosting_info": {
                    "hosting_mac": "fa:16:3e:4f:33:fb",
                    "hosting_port_id": "3f70129a-aefd-47ee-94d7-06ce9abf3cba",
                    "hosting_port_name": "",
                    "physical_interface": "Port-channel10",
                    "segmentation_id": 2577
                },
                "id": "3f70129a-aefd-47ee-94d7-06ce9abf3cba",
                "mac_address": "fa:16:3e:4f:33:fb",
                "name": "",
                "network_id": "08aedf4f-dd83-4c1c-84f5-759430331eae",
                "security_groups": [],
                "status": "ACTIVE",
                "subnets": [
                    {
                        "cidr": "20.20.20.0/24",
                        "gateway_ip": "20.20.20.1",
                        "id": "90c10f81-47f2-487c-ad1b-c7212c378e08",
                        "ipv6_ra_mode": None
                    }
                ],
                "tenant_id": ""
            }
        ],
        "admin_state_up": True,
        "cisco_ha:details": {
            "priority": 100,
            "probe_connectivity": False,
            "redundancy_level": 1,
            "redundancy_routers": [
                {
                    "id": "e7f8b39b-2d87-4eb7-9102-fd24eb3a2416",
                    "priority": 97,
                    "state": "STANDBY"
                }
            ],
            "state": "ACTIVE",
            "type": "HSRP"
        },
        "cisco_ha:enabled": True,
        "external_gateway_info": {
            "external_fixed_ips": [
                {
                    "ip_address": "172.16.0.116",
                    "subnet_id": "8f9c885d-a2ad-4fff-8127-2d4b3af6015b"
                }
            ],
            "network_id": "7a38a5fd-3ad9-4952-ab08-4def22407269"
        },
        "gw_port": {
            "admin_state_up": True,
            "allowed_address_pairs": [],
            "binding:host_id": "asr1k-k7-controller-1-R2",
            "binding:profile": {},
            "binding:vif_details": {
                "ovs_hybrid_plug": True,
                "port_filter": True
            },
            "binding:vif_type": "ovs",
            "binding:vnic_type": "normal",
            "device_id": "bdc5b513-7fee-4fe0-9514-78602246d6ee",
            "device_owner": "network:router_gateway",
            "extra_dhcp_opts": [],
            "extra_subnets": [],
            "fixed_ips": [
                {
                    "ip_address": "172.16.0.116",
                    "prefixlen": 16,
                    "subnet_id": "8f9c885d-a2ad-4fff-8127-2d4b3af6015b"
                }
            ],
            "ha_info": {
                "group": "1064",
                "ha_port": {
                    "admin_state_up": True,
                    "allowed_address_pairs": [],
                    "binding:host_id": "asr1k-k7-controller-1-R2",
                    "binding:profile": {},
                    "binding:vif_details": {
                        "ovs_hybrid_plug": True,
                        "port_filter": True
                    },
                    "binding:vif_type": "ovs",
                    "binding:vnic_type": "normal",
                    "device_id": "bdc5b513-7fee-4fe0-9514-78602246d6ee",
                    "device_owner": "network:router_gateway",
                    "extra_dhcp_opts": [],
                    "extra_subnets": [],
                    "fixed_ips": [
                        {
                            "ip_address": "172.16.0.116",
                            "prefixlen": 16,
                            "subnet_id": "8f9c885d-a2ad-4fff-8127-2d4b3af6015b"
                        }
                    ],
                    "id": "bb32ab93-88e7-4c80-aeec-387703cba0ea",
                    "mac_address": "fa:16:3e:ef:c0:88",
                    "name": "",
                    "network_id": "7a38a5fd-3ad9-4952-ab08-4def22407269",
                    "security_groups": [],
                    "status": "ACTIVE",
                    "subnets": [
                        {
                            "cidr": "172.16.0.0/16",
                            "gateway_ip": "172.16.0.1",
                            "id": "8f9c885d-a2ad-4fff-8127-2d4b3af6015b",
                            "ipv6_ra_mode": None
                        }
                    ],
                    "tenant_id": ""
                },
                "other_config": "",
                "timers_config": "",
                "tracking_config": "",
                "type": "HSRP"
            },
            "hosting_info": {
                "hosting_mac": "fa:16:3e:ef:c0:88",
                "hosting_port_id": "bb32ab93-88e7-4c80-aeec-387703cba0ea",
                "hosting_port_name": "",
                "physical_interface": "Port-channel10",
                "segmentation_id": 3000
            },
            "id": "bb32ab93-88e7-4c80-aeec-387703cba0ea",
            "mac_address": "fa:16:3e:ef:c0:88",
            "name": "",
            "network_id": "7a38a5fd-3ad9-4952-ab08-4def22407269",
            "security_groups": [],
            "status": "ACTIVE",
            "subnets": [
                {
                    "cidr": "172.16.0.0/16",
                    "gateway_ip": "172.16.0.1",
                    "id": "8f9c885d-a2ad-4fff-8127-2d4b3af6015b",
                    "ipv6_ra_mode": None
                }
            ],
            "tenant_id": ""
        },
        "gw_port_id": "bb32ab93-88e7-4c80-aeec-387703cba0ea",
        "hosting_device": {
            "admin_state_up": True,
            "booting_time": 360,
            "created_at": "2015-11-16 22:54:18",
            "credentials": {
                "password": "cisco123",
                "user_name": "admin"
            },
            "host_category": "Hardware",
            "id": "00000000-0000-0000-0000-000000000003",
            "management_ip_address": "172.20.231.19",
            "name": "ASR1k template",
            "protocol_port": 22,
            "service_types": "router:FW:VPN",
            "status": "ACTIVE",
            "template_id": "00000000-0000-0000-0000-000000000003",
            "timeout": None
        },
        "id": "bdc5b513-7fee-4fe0-9514-78602246d6ee",
        "name": "ROUTER1",
        "router_type": {
            "cfg_agent_driver": "neutron.plugins.cisco.cfg_agent."
                                "device_drivers.asr1k.asr1k_routing_driver."
                                "ASR1kRoutingDriver",
            "cfg_agent_service_helper": "neutron.plugins.cisco.cfg_agent."
                                        "service_helpers.routing_svc_helper."
                                        "RoutingServiceHelper",
            "id": "00000000-0000-0000-0000-000000000003",
            "name": "ASR1k_router"
        },
        "routerhost:hosting_device": "00000000-0000-0000-0000-000000000003",
        "routerrole:role": None,
        "routertype-aware-scheduler:auto_schedule": True,
        "routertype-aware-scheduler:share_hosting_device": True,
        "routertype:id": "00000000-0000-0000-0000-000000000003",
        "routes": [],
        "share_host": True,
        "status": "ACTIVE",
        "tenant_id": "2386c5d814814fb68b037ab7aec784f8"
    },
    {
        "_floatingips": [
            {
                "fixed_ip_address": "10.2.0.5",
                "floating_ip_address": "172.16.0.126",
                "floating_network_id": "7a38a5fd-3ad9-4952-ab08-4def22407269",
                "id": "0b38bc64-7b01-440d-94fa-4e967f410aa6",
                "port_id": "c186b8d5-a835-4128-a7e0-20b273defc02",
                "router_id": "17478180-082c-4dbc-a35b-e7fdfb260cd6",
                "status": "ACTIVE",
                "tenant_id": "ec3a24a6ffa244aab77501f5cea69be9"
            }
        ],
        "_interfaces": [
            {
                "admin_state_up": True,
                "allowed_address_pairs": [],
                "binding:host_id": "asr1k-k7-controller-1-R2",
                "binding:profile": {},
                "binding:vif_details": {
                    "ovs_hybrid_plug": True,
                    "port_filter": True
                },
                "binding:vif_type": "ovs",
                "binding:vnic_type": "normal",
                "device_id": "3ea5f9f9-0ecf-4595-b8be-3f340f2c82e3",
                "device_owner": "network:router_interface",
                "extra_dhcp_opts": [],
                "extra_subnets": [],
                "fixed_ips": [
                    {
                        "ip_address": "10.2.0.4",
                        "prefixlen": 24,
                        "subnet_id": "eee64e1c-e7e8-4b06-9161-148ca25bb04d"
                    }
                ],
                "ha_info": {
                    "group": "1064",
                    "ha_port": {
                        "admin_state_up": True,
                        "allowed_address_pairs": [],
                        "binding:host_id": "",
                        "binding:profile": {},
                        "binding:vif_details": {},
                        "binding:vif_type": "unbound",
                        "binding:vnic_type": "normal",
                        "device_id": "17478180-082c-4dbc-a35b-e7fdfb260cd6",
                        "device_owner": "network:router_interface",
                        "extra_dhcp_opts": [],
                        "extra_subnets": [],
                        "fixed_ips": [
                            {
                                "ip_address": "10.2.0.1",
                                "prefixlen": 24,
                                "subnet_id": "eee64e1c-e7e8-4b06-9161"
                                             "-148ca25bb04d"
                            }
                        ],
                        "id": "f1d20abe-b7ec-4f14-91f5-3465e1b0fd6b",
                        "mac_address": "fa:16:3e:1c:d5:40",
                        "name": "",
                        "network_id": "b26c62fa-3fa8-4615-ad8d-bf7bdd443712",
                        "security_groups": [],
                        "status": "DOWN",
                        "subnets": [
                            {
                                "cidr": "10.2.0.0/24",
                                "gateway_ip": "10.2.0.1",
                                "id": "eee64e1c-e7e8-4b06-9161-148ca25bb04d",
                                "ipv6_ra_mode": None
                            }
                        ],
                        "tenant_id": "ec3a24a6ffa244aab77501f5cea69be9"
                    },
                    "other_config": "",
                    "timers_config": "",
                    "tracking_config": "",
                    "type": "HSRP"
                },
                "hosting_info": {
                    "hosting_mac": "fa:16:3e:69:50:00",
                    "hosting_port_id": "47f1a63e-2c75-4975-aeac-e1f69b9904b6",
                    "hosting_port_name": "",
                    "physical_interface": "Port-channel10",
                    "segmentation_id": 2564
                },
                "id": "47f1a63e-2c75-4975-aeac-e1f69b9904b6",
                "mac_address": "fa:16:3e:69:50:00",
                "name": "",
                "network_id": "b26c62fa-3fa8-4615-ad8d-bf7bdd443712",
                "security_groups": [],
                "status": "ACTIVE",
                "subnets": [
                    {
                        "cidr": "10.2.0.0/24",
                        "gateway_ip": "10.2.0.1",
                        "id": "eee64e1c-e7e8-4b06-9161-148ca25bb04d",
                        "ipv6_ra_mode": None
                    }
                ],
                "tenant_id": ""
            }
        ],
        "admin_state_up": True,
        "cisco_ha:details": {
            "priority": 100,
            "probe_connectivity": False,
            "redundancy_level": 1,
            "redundancy_routers": [
                {
                    "id": "3ea5f9f9-0ecf-4595-b8be-3f340f2c82e3",
                    "priority": 97,
                    "state": "STANDBY"
                }
            ],
            "state": "ACTIVE",
            "type": "HSRP"
        },
        "cisco_ha:enabled": True,
        "external_gateway_info": {
            "external_fixed_ips": [
                {
                    "ip_address": "172.16.0.125",
                    "subnet_id": "8f9c885d-a2ad-4fff-8127-2d4b3af6015b"
                }
            ],
            "network_id": "7a38a5fd-3ad9-4952-ab08-4def22407269"
        },
        "gw_port": {
            "admin_state_up": True,
            "allowed_address_pairs": [],
            "binding:host_id": "asr1k-k7-controller-1-R2",
            "binding:profile": {},
            "binding:vif_details": {
                "ovs_hybrid_plug": True,
                "port_filter": True
            },
            "binding:vif_type": "ovs",
            "binding:vnic_type": "normal",
            "device_id": "3ea5f9f9-0ecf-4595-b8be-3f340f2c82e3",
            "device_owner": "network:router_gateway",
            "extra_dhcp_opts": [],
            "extra_subnets": [],
            "fixed_ips": [
                {
                    "ip_address": "172.16.0.125",
                    "prefixlen": 16,
                    "subnet_id": "8f9c885d-a2ad-4fff-8127-2d4b3af6015b"
                }
            ],
            "ha_info": {
                "group": "1064",
                "ha_port": {
                    "admin_state_up": True,
                    "allowed_address_pairs": [],
                    "binding:host_id": "asr1k-k7-controller-1-R2",
                    "binding:profile": {},
                    "binding:vif_details": {
                        "ovs_hybrid_plug": True,
                        "port_filter": True
                    },
                    "binding:vif_type": "ovs",
                    "binding:vnic_type": "normal",
                    "device_id": "17478180-082c-4dbc-a35b-e7fdfb260cd6",
                    "device_owner": "network:router_gateway",
                    "extra_dhcp_opts": [],
                    "extra_subnets": [],
                    "fixed_ips": [
                        {
                            "ip_address": "172.16.0.124",
                            "prefixlen": 16,
                            "subnet_id": "8f9c885d-a2ad-4fff-8127-2d4b3af6015b"
                        }
                    ],
                    "id": "b7f1d6eb-7016-4bcd-93d5-9f5254bd672e",
                    "mac_address": "fa:16:3e:01:8a:85",
                    "name": "",
                    "network_id": "7a38a5fd-3ad9-4952-ab08-4def22407269",
                    "security_groups": [],
                    "status": "ACTIVE",
                    "subnets": [
                        {
                            "cidr": "172.16.0.0/16",
                            "gateway_ip": "172.16.0.1",
                            "id": "8f9c885d-a2ad-4fff-8127-2d4b3af6015b",
                            "ipv6_ra_mode": None
                        }
                    ],
                    "tenant_id": ""
                },
                "other_config": "",
                "timers_config": "",
                "tracking_config": "",
                "type": "HSRP"
            },
            "hosting_info": {
                "hosting_mac": "fa:16:3e:4c:47:f2",
                "hosting_port_id": "7d71cf06-2463-4350-a5ee-a8bb09c5466c",
                "hosting_port_name": "",
                "physical_interface": "Port-channel10",
                "segmentation_id": 3000
            },
            "id": "7d71cf06-2463-4350-a5ee-a8bb09c5466c",
            "mac_address": "fa:16:3e:4c:47:f2",
            "name": "",
            "network_id": "7a38a5fd-3ad9-4952-ab08-4def22407269",
            "security_groups": [],
            "status": "ACTIVE",
            "subnets": [
                {
                    "cidr": "172.16.0.0/16",
                    "gateway_ip": "172.16.0.1",
                    "id": "8f9c885d-a2ad-4fff-8127-2d4b3af6015b",
                    "ipv6_ra_mode": None
                }
            ],
            "tenant_id": ""
        },
        "gw_port_id": "7d71cf06-2463-4350-a5ee-a8bb09c5466c",
        "hosting_device": {
            "admin_state_up": True,
            "booting_time": 360,
            "created_at": "2015-11-16 22:54:18",
            "credentials": {
                "password": "cisco123",
                "user_name": "admin"
            },
            "host_category": "Hardware",
            "id": "00000000-0000-0000-0000-000000000003",
            "management_ip_address": "172.20.231.19",
            "name": "ASR1k template",
            "protocol_port": 22,
            "service_types": "router:FW:VPN",
            "status": "ACTIVE",
            "template_id": "00000000-0000-0000-0000-000000000003",
            "timeout": None
        },
        "id": "3ea5f9f9-0ecf-4595-b8be-3f340f2c82e3",
        "name": "rally_net_vm4FfgSGedr94BsU_HA_backup_1",
        "router_type": {
            "cfg_agent_driver": "neutron.plugins.cisco.cfg_agent."
                                "device_drivers.asr1k.asr1k_routing_driver."
                                "ASR1kRoutingDriver",
            "cfg_agent_service_helper": "neutron.plugins.cisco.cfg_agent."
                                        "service_helpers.routing_svc_helper."
                                        "RoutingServiceHelper",
            "id": "00000000-0000-0000-0000-000000000003",
            "name": "ASR1k_router"
        },
        "routerhost:hosting_device": "00000000-0000-0000-0000-000000000003",
        "routerrole:role": "HA-Redundancy",
        "routertype-aware-scheduler:auto_schedule": True,
        "routertype-aware-scheduler:share_hosting_device": True,
        "routertype:id": "00000000-0000-0000-0000-000000000003",
        "routes": [],
        "share_host": True,
        "status": "ACTIVE",
        "tenant_id": ""
    },
    {
        "_floatingips": [
            {
                "fixed_ip_address": "192.168.0.9",
                "floating_ip_address": "172.16.0.109",
                "floating_network_id": "7a38a5fd-3ad9-4952-ab08-4def22407269",
                "id": "0e6807b0-0de8-4df2-bd9d-6c822d512dcc",
                "port_id": "b488932b-c0dd-40ee-bc84-a35a01e3c967",
                "router_id": "92740eb5-3604-421a-91a3-04561af775fc",
                "status": "ACTIVE",
                "tenant_id": "ecf8697e8dc04fef8be7c7f9c3594de9"
            }
        ],
        "_interfaces": [
            {
                "admin_state_up": True,
                "allowed_address_pairs": [],
                "binding:host_id": "asr1k-k7-controller-1-R2",
                "binding:profile": {},
                "binding:vif_details": {
                    "ovs_hybrid_plug": True,
                    "port_filter": True
                },
                "binding:vif_type": "ovs",
                "binding:vnic_type": "normal",
                "device_id": "0562d6c6-0732-4d68-9eb2-e423e7bef004",
                "device_owner": "network:router_interface",
                "extra_dhcp_opts": [],
                "extra_subnets": [],
                "fixed_ips": [
                    {
                        "ip_address": "192.168.0.7",
                        "prefixlen": 24,
                        "subnet_id": "6fdaaae3-4034-4890-ab60-1411527e4556"
                    }
                ],
                "ha_info": {
                    "group": "1064",
                    "ha_port": {
                        "admin_state_up": True,
                        "allowed_address_pairs": [],
                        "binding:host_id": "",
                        "binding:profile": {},
                        "binding:vif_details": {},
                        "binding:vif_type": "unbound",
                        "binding:vnic_type": "normal",
                        "device_id": "92740eb5-3604-421a-91a3-04561af775fc",
                        "device_owner": "network:router_interface",
                        "extra_dhcp_opts": [],
                        "extra_subnets": [],
                        "fixed_ips": [
                            {
                                "ip_address": "192.168.0.1",
                                "prefixlen": 24,
                                "subnet_id": "6fdaaae3-4034-4890-ab60-"
                                             "1411527e4556"
                            }
                        ],
                        "id": "4f4be053-1ee1-4193-a0c4-d03dce548986",
                        "mac_address": "fa:16:3e:79:67:80",
                        "name": "",
                        "network_id": "5ca17eaa-8761-48b3-9284-983d9c0983df",
                        "security_groups": [],
                        "status": "DOWN",
                        "subnets": [
                            {
                                "cidr": "192.168.0.0/24",
                                "gateway_ip": "192.168.0.1",
                                "id": "6fdaaae3-4034-4890-ab60-1411527e4556",
                                "ipv6_ra_mode": None
                            }
                        ],
                        "tenant_id": "ecf8697e8dc04fef8be7c7f9c3594de9"
                    },
                    "other_config": "",
                    "timers_config": "",
                    "tracking_config": "",
                    "type": "HSRP"
                },
                "hosting_info": {
                    "hosting_mac": "fa:16:3e:e8:a3:24",
                    "hosting_port_id": "86f655d1-49b4-44c9-b91e-ec683be80ce8",
                    "hosting_port_name": "",
                    "physical_interface": "Port-channel10",
                    "segmentation_id": 2535
                },
                "id": "86f655d1-49b4-44c9-b91e-ec683be80ce8",
                "mac_address": "fa:16:3e:e8:a3:24",
                "name": "",
                "network_id": "5ca17eaa-8761-48b3-9284-983d9c0983df",
                "security_groups": [],
                "status": "ACTIVE",
                "subnets": [
                    {
                        "cidr": "192.168.0.0/24",
                        "gateway_ip": "192.168.0.1",
                        "id": "6fdaaae3-4034-4890-ab60-1411527e4556",
                        "ipv6_ra_mode": None
                    }
                ],
                "tenant_id": ""
            }
        ],
        "admin_state_up": True,
        "cisco_ha:details": {
            "priority": 100,
            "probe_connectivity": False,
            "redundancy_level": 1,
            "redundancy_routers": [
                {
                    "id": "ea0f1fde-e00f-49f1-aa60-6e30c1197ea5",
                    "priority": 97,
                    "state": "STANDBY"
                }
            ],
            "state": "ACTIVE",
            "type": "HSRP"
        },
        "cisco_ha:enabled": True,
        "external_gateway_info": {
            "external_fixed_ips": [
                {
                    "ip_address": "172.16.0.101",
                    "subnet_id": "8f9c885d-a2ad-4fff-8127-2d4b3af6015b"
                }
            ],
            "network_id": "7a38a5fd-3ad9-4952-ab08-4def22407269"
        },
        "gw_port": {
            "admin_state_up": True,
            "allowed_address_pairs": [],
            "binding:host_id": "asr1k-k7-controller-1-R2",
            "binding:profile": {},
            "binding:vif_details": {
                "ovs_hybrid_plug": True,
                "port_filter": True
            },
            "binding:vif_type": "ovs",
            "binding:vnic_type": "normal",
            "device_id": "92740eb5-3604-421a-91a3-04561af775fc",
            "device_owner": "network:router_gateway",
            "extra_dhcp_opts": [],
            "extra_subnets": [],
            "fixed_ips": [
                {
                    "ip_address": "172.16.0.101",
                    "prefixlen": 16,
                    "subnet_id": "8f9c885d-a2ad-4fff-8127-2d4b3af6015b"
                }
            ],
            "ha_info": {
                "group": "1064",
                "ha_port": {
                    "admin_state_up": True,
                    "allowed_address_pairs": [],
                    "binding:host_id": "asr1k-k7-controller-1-R2",
                    "binding:profile": {},
                    "binding:vif_details": {
                        "ovs_hybrid_plug": True,
                        "port_filter": True
                    },
                    "binding:vif_type": "ovs",
                    "binding:vnic_type": "normal",
                    "device_id": "92740eb5-3604-421a-91a3-04561af775fc",
                    "device_owner": "network:router_gateway",
                    "extra_dhcp_opts": [],
                    "extra_subnets": [],
                    "fixed_ips": [
                        {
                            "ip_address": "172.16.0.101",
                            "prefixlen": 16,
                            "subnet_id": "8f9c885d-a2ad-4fff-8127-2d4b3af6015b"
                        }
                    ],
                    "id": "f8695a3d-28cd-4833-aecd-b5b64cb7f7a9",
                    "mac_address": "fa:16:3e:96:ed:91",
                    "name": "",
                    "network_id": "7a38a5fd-3ad9-4952-ab08-4def22407269",
                    "security_groups": [],
                    "status": "ACTIVE",
                    "subnets": [
                        {
                            "cidr": "172.16.0.0/16",
                            "gateway_ip": "172.16.0.1",
                            "id": "8f9c885d-a2ad-4fff-8127-2d4b3af6015b",
                            "ipv6_ra_mode": None
                        }
                    ],
                    "tenant_id": ""
                },
                "other_config": "",
                "timers_config": "",
                "tracking_config": "",
                "type": "HSRP"
            },
            "hosting_info": {
                "hosting_mac": "fa:16:3e:96:ed:91",
                "hosting_port_id": "f8695a3d-28cd-4833-aecd-b5b64cb7f7a9",
                "hosting_port_name": "",
                "physical_interface": "Port-channel10",
                "segmentation_id": 3000
            },
            "id": "f8695a3d-28cd-4833-aecd-b5b64cb7f7a9",
            "mac_address": "fa:16:3e:96:ed:91",
            "name": "",
            "network_id": "7a38a5fd-3ad9-4952-ab08-4def22407269",
            "security_groups": [],
            "status": "ACTIVE",
            "subnets": [
                {
                    "cidr": "172.16.0.0/16",
                    "gateway_ip": "172.16.0.1",
                    "id": "8f9c885d-a2ad-4fff-8127-2d4b3af6015b",
                    "ipv6_ra_mode": None
                }
            ],
            "tenant_id": ""
        },
        "gw_port_id": "f8695a3d-28cd-4833-aecd-b5b64cb7f7a9",
        "hosting_device": {
            "admin_state_up": True,
            "booting_time": 360,
            "created_at": "2015-11-16 22:54:18",
            "credentials": {
                "password": "cisco123",
                "user_name": "admin"
            },
            "host_category": "Hardware",
            "id": "00000000-0000-0000-0000-000000000003",
            "management_ip_address": "172.20.231.19",
            "name": "ASR1k template",
            "protocol_port": 22,
            "service_types": "router:FW:VPN",
            "status": "ACTIVE",
            "template_id": "00000000-0000-0000-0000-000000000003",
            "timeout": None
        },
        "id": "92740eb5-3604-421a-91a3-04561af775fc",
        "name": "ROUTER-1",
        "router_type": {
            "cfg_agent_driver": "neutron.plugins.cisco.cfg_agent."
                                "device_drivers.asr1k.asr1k_routing_driver."
                                "ASR1kRoutingDriver",
            "cfg_agent_service_helper": "neutron.plugins.cisco.cfg_agent."
                                        "service_helpers.routing_svc_helper."
                                        "RoutingServiceHelper",
            "id": "00000000-0000-0000-0000-000000000003",
            "name": "ASR1k_router"
        },
        "routerhost:hosting_device": "00000000-0000-0000-0000-000000000003",
        "routerrole:role": None,
        "routertype-aware-scheduler:auto_schedule": True,
        "routertype-aware-scheduler:share_hosting_device": True,
        "routertype:id": "00000000-0000-0000-0000-000000000003",
        "routes": [],
        "share_host": True,
        "status": "ACTIVE",
        "tenant_id": "ecf8697e8dc04fef8be7c7f9c3594de9"
    },
    {
        "_interfaces": [
            {
                "admin_state_up": True,
                "allowed_address_pairs": [],
                "binding:host_id": "asr1k-k7-controller-1-R2",
                "binding:profile": {},
                "binding:vif_details": {
                    "ovs_hybrid_plug": True,
                    "port_filter": True
                },
                "binding:vif_type": "ovs",
                "binding:vnic_type": "normal",
                "device_id": "213fd355-78c3-44cc-bd6e-492b9ade2602",
                "device_owner": "network:router_interface",
                "extra_dhcp_opts": [],
                "extra_subnets": [],
                "fixed_ips": [
                    {
                        "ip_address": "172.16.0.105",
                        "prefixlen": 16,
                        "subnet_id": "8f9c885d-a2ad-4fff-8127-2d4b3af6015b"
                    }
                ],
                "ha_info": {
                    "group": "1064",
                    "ha_port": {
                        "admin_state_up": True,
                        "allowed_address_pairs": [],
                        "binding:host_id": "",
                        "binding:profile": {},
                        "binding:vif_details": {},
                        "binding:vif_type": "unbound",
                        "binding:vnic_type": "normal",
                        "device_id": "a2617b46-78ef-49ba-95b5-0f3ff90f1f91",
                        "device_owner": "network:router_interface",
                        "extra_dhcp_opts": [],
                        "extra_subnets": [],
                        "fixed_ips": [{
                            "ip_address": "172.16.0.104",
                            "prefixlen": 16,
                            "subnet_id": "8f9c885d-a2ad-4fff-8127-2d4b3af6015b"
                        }],
                        "id": "b768491c-4b76-492f-a0ee-56bc50867a07",
                        "mac_address": "fa:16:3e:77:ae:bb",
                        "name": "",
                        "network_id": "7a38a5fd-3ad9-4952-ab08-4def22407269",
                        "security_groups": [],
                        "status": "DOWN",
                        "subnets": [
                            {
                                "cidr": "172.16.0.0/16",
                                "gateway_ip": "172.16.0.1",
                                "id": "8f9c885d-a2ad-4fff-8127-2d4b3af6015b",
                                "ipv6_ra_mode": None
                            }
                        ],
                        "tenant_id": ""
                    },
                    "other_config": "",
                    "timers_config": "",
                    "tracking_config": "",
                    "type": "HSRP"
                },
                "hosting_info": {
                    "hosting_mac": "fa:16:3e:97:ec:f2",
                    "hosting_port_id": "527b7bc0-d6ba-4d07-932a-60e96da86173",
                    "hosting_port_name": "",
                    "physical_interface": "Port-channel10",
                    "segmentation_id": 3000
                },
                "id": "527b7bc0-d6ba-4d07-932a-60e96da86173",
                "mac_address": "fa:16:3e:97:ec:f2",
                "name": "",
                "network_id": "7a38a5fd-3ad9-4952-ab08-4def22407269",
                "security_groups": [],
                "status": "ACTIVE",
                "subnets": [
                    {
                        "cidr": "172.16.0.0/16",
                        "gateway_ip": "172.16.0.1",
                        "id": "8f9c885d-a2ad-4fff-8127-2d4b3af6015b",
                        "ipv6_ra_mode": None
                    }
                ],
                "tenant_id": ""
            }
        ],
        "admin_state_up": True,
        "cisco_ha:details": {
            "priority": 100,
            "probe_connectivity": False,
            "redundancy_level": 2,
            "redundancy_routers": [
                {
                    "id": "213fd355-78c3-44cc-bd6e-492b9ade2602",
                    "priority": 97,
                    "state": "STANDBY"
                },
                {
                    "id": "15315cc4-d00d-446d-bf6e-b077faaefaa1",
                    "priority": 100,
                    "state": "STANDBY"
                }
            ],
            "state": "ACTIVE",
            "type": "HSRP"
        },
        "cisco_ha:enabled": True,
        "external_gateway_info": {
            "external_fixed_ips": [
                {
                    "ip_address": "172.16.0.105",
                    "subnet_id": "8f9c885d-a2ad-4fff-8127-2d4b3af6015b"
                }
            ],
            "network_id": "7a38a5fd-3ad9-4952-ab08-4def22407269"
        },
        "gw_port_id": None,
        "hosting_device": {
            "admin_state_up": True,
            "booting_time": 360,
            "created_at": "2015-11-16 22:54:18",
            "credentials": {
                "password": "cisco123",
                "user_name": "admin"
            },
            "host_category": "Hardware",
            "id": "00000000-0000-0000-0000-000000000003",
            "management_ip_address": "172.20.231.19",
            "name": "ASR1k template",
            "protocol_port": 22,
            "service_types": "router:FW:VPN",
            "status": "ACTIVE",
            "template_id": "00000000-0000-0000-0000-000000000003",
            "timeout": None
        },
        "id": "213fd355-78c3-44cc-bd6e-492b9ade2602",
        "name": "Global-router-0000-000000000003",
        "router_type": {
            "cfg_agent_driver": "neutron.plugins.cisco.cfg_agent."
                                "device_drivers.asr1k.asr1k_routing_driver."
                                "ASR1kRoutingDriver",
            "cfg_agent_service_helper": "neutron.plugins.cisco.cfg_agent."
                                        "service_helpers.routing_svc_helper."
                                        "RoutingServiceHelper",
            "id": "00000000-0000-0000-0000-000000000003",
            "name": "ASR1k_router"
        },
        "routerhost:hosting_device": "00000000-0000-0000-0000-000000000003",
        "routerrole:role": "Global",
        "routertype-aware-scheduler:auto_schedule": False,
        "routertype-aware-scheduler:share_hosting_device": True,
        "routertype:id": "00000000-0000-0000-0000-000000000003",
        "routes": [],
        "share_host": True,
        "status": "ACTIVE",
        "tenant_id": ""
    }
]


class ASR1kCfgSyncer(base.BaseTestCase):

    def setUp(self):
        super(ASR1kCfgSyncer, self).setUp()

        # self._read_neutron_db_data()
        self.router_db_info = NEUTRON_DB
        self.hosting_device_info = {
            'id': '00000000-0000-0000-0000-000000000003'}
        self.driver = mock.Mock()
        self.config_syncer = asr1k_cfg_syncer.ConfigSyncer(
            self.router_db_info, self.driver, self.hosting_device_info)

    def test_delete_invalid_cfg_empty_routers_list(self):
        """
        expected invalid_cfg::

            [u'ip nat inside source static 10.2.0.5 172.16.0.126 vrf'
              ' nrouter-3ea5f9 redundancy neutron-hsrp-1064-3000',
             u'ip nat inside source list neutron_acl_2564 pool'
              ' nrouter-3ea5f9_nat_pool vrf nrouter-3ea5f9 overload',
             u'ip nat pool nrouter-3ea5f9_nat_pool 172.16.0.124'
              ' 172.16.0.124 netmask 255.255.0.0',
             u'ip route vrf nrouter-3ea5f9 0.0.0.0 0.0.0.0'
              ' Port-channel10.3000 172.16.0.1',
             u'ip access-list standard neutron_acl_2564',
             <IOSCfgLine # 83 'interface Port-channel10.2564'>,
             <IOSCfgLine # 96 'interface Port-channel10.3000'>,
             u'nrouter-3ea5f9']
        """

        cfg.CONF.set_override('enable_multi_region', False, 'multi_region')

        # simulate a blank neutron-db
        router_db_info = []

        self.config_syncer = asr1k_cfg_syncer.ConfigSyncer(router_db_info,
                                                      self.driver,
                                                      self.hosting_device_info)
        self.config_syncer.get_running_config = mock.Mock(
            return_value=ASR_BASIC_RUNNING_CFG_NO_MULTI_REGION)

        invalid_cfg = self.config_syncer.delete_invalid_cfg()
        self.assertEqual(8, len(invalid_cfg))

    def test_delete_invalid_cfg_with_multi_region_and_empty_routers_list(self):
        """
        This test verifies that the  cfg-syncer will delete invalid cfg
        if the neutron-db (routers dictionary list) happens to be empty.

        Since the neutron-db router_db_info is empty, all region 0000002
        running-config should be deleted.

        Expect 8 invalid configs found::

            ['ip nat inside source static 10.2.0.5 172.16.0.126 '
             'vrf nrouter-3ea5f9-0000002 redundancy neutron-hsrp-1064-3000',
             'ip nat inside source list neutron_acl_0000002_2564 pool '
             'nrouter-3ea5f9-0000002_nat_pool vrf nrouter-3ea5f9-0000002 '
             'overload',
             'ip nat pool nrouter-3ea5f9-0000002_nat_pool '
             '172.16.0.124 172.16.0.124 netmask 255.255.0.0',
             'ip route vrf nrouter-3ea5f9-0000002 0.0.0.0 0.0.0.0 '
             'Port-channel10.3000 172.16.0.1',
             'ip access-list standard neutron_acl_0000002_2564',
             <IOSCfgLine # 83 'interface Port-channel10.2564'>,
             <IOSCfgLine # 96 'interface Port-channel10.3000'>,
             'nrouter-3ea5f9-0000002']
        """

        cfg.CONF.set_override('enable_multi_region', True, 'multi_region')
        cfg.CONF.set_override('region_id', '0000002', 'multi_region')
        cfg.CONF.set_override('other_region_ids', ['0000001'], 'multi_region')

        # simulate a blank neutron-db
        router_db_info = []
        self.config_syncer = asr1k_cfg_syncer.ConfigSyncer(router_db_info,
                                                      self.driver,
                                                      self.hosting_device_info)

        self.config_syncer.get_running_config = mock.Mock(
            return_value=ASR_BASIC_RUNNING_CFG)

        invalid_cfg = self.config_syncer.delete_invalid_cfg()

        self.assertEqual(8, len(invalid_cfg))

    def test_clean_interfaces_basic_multi_region_enabled(self):
        """
        In this test, we are simulating a cfg-sync, clean_interfaces for
        region 0000002 cfg-agent.  Running-cfg only exists for region
        0000001.

        At the end of test, we should expect zero entries in invalid_cfg.
        """

        cfg.CONF.set_override('enable_multi_region', True, 'multi_region')
        cfg.CONF.set_override('region_id', '0000002', 'multi_region')
        cfg.CONF.set_override('other_region_ids', ['0000001'], 'multi_region')

        intf_segment_dict = self.config_syncer.intf_segment_dict
        segment_nat_dict = self.config_syncer.segment_nat_dict

        conn = self.driver._get_connection()

        parsed_cfg = HTParser(ASR_RUNNING_CFG_NO_R2_JSON_JUST_INTERFACES)

        invalid_cfg = self.config_syncer.clean_interfaces(conn,
                                              intf_segment_dict,
                                              segment_nat_dict,
                                              parsed_cfg)
        self.assertEqual([], invalid_cfg)

    def test_clean_interfaces_multi_region_disabled(self):
        """
        In this test, we are simulating a cfg-sync, clean_interfaces for
        region 0000002 cfg-agent.  Running-cfg only exists for region
        0000001, but multi_region is disabled.

        At the end of test, we should expect zero entries in invalid_cfg.
        """
        cfg.CONF.set_override('enable_multi_region', False, 'multi_region')

        intf_segment_dict = self.config_syncer.intf_segment_dict
        segment_nat_dict = self.config_syncer.segment_nat_dict

        conn = self.driver._get_connection()

        parsed_cfg = HTParser(ASR_RUNNING_CFG_NO_R2_JSON_JUST_INTERFACES)

        invalid_cfg = self.config_syncer.clean_interfaces(conn,
                                              intf_segment_dict,
                                              segment_nat_dict,
                                              parsed_cfg)
        self.assertEqual([], invalid_cfg)

    def test_clean_interfaces_R2_run_cfg_present_multi_region_enabled(self):
        """
        In this test, we are simulating a cfg-sync, clean_interfaces for
        region 0000002 cfg-agent.  Existing running-cfg exists for region
        0000001 and 0000002.

        At the end of test, we should expect zero entries in invalid_cfg.
        """
        cfg.CONF.set_override('enable_multi_region', True, 'multi_region')
        cfg.CONF.set_override('region_id', '0000002', 'multi_region')
        cfg.CONF.set_override('other_region_ids', ['0000001'], 'multi_region')

        intf_segment_dict = self.config_syncer.intf_segment_dict
        segment_nat_dict = self.config_syncer.segment_nat_dict

        conn = self.driver._get_connection()

        parsed_cfg = HTParser(ASR_BASIC_RUNNING_CFG)

        invalid_cfg = self.config_syncer.clean_interfaces(conn,
                                              intf_segment_dict,
                                              segment_nat_dict,
                                              parsed_cfg)
        self.assertEqual([], invalid_cfg)

    def test_clean_interfaces_R2_with_invalid_intfs(self):
        """
        In this test, we are simulating a cfg-sync, clean_interfaces for
        region 0000002 cfg-agent.  Existing running-cfg exists for region
        0000001 and 0000002.

        At the end of test, we should expect two invalid intfs
        detected.

        invalid tenant router, int Po10.2536 (invalid segment-id)
        invalid ext-gw-port, int Po10.3000 (invalid HSRP VIP)
        """
        cfg.CONF.set_override('enable_multi_region', True, 'multi_region')
        cfg.CONF.set_override('region_id', '0000002', 'multi_region')
        cfg.CONF.set_override('other_region_ids', ['0000001'], 'multi_region')

        intf_segment_dict = self.config_syncer.intf_segment_dict
        segment_nat_dict = self.config_syncer.segment_nat_dict

        conn = self.driver._get_connection()
        parsed_cfg = HTParser(ASR_RUNNING_CFG_WITH_INVALID_INTFS)

        invalid_cfg = self.config_syncer.clean_interfaces(conn,
                                              intf_segment_dict,
                                              segment_nat_dict,
                                              parsed_cfg)

        self.assertEqual(2, len(invalid_cfg))

    def test_clean_acls_basic_running_cfg(self):
        """
        region 1 acls should be ignored
        """
        cfg.CONF.set_override('enable_multi_region', True, 'multi_region')
        cfg.CONF.set_override('region_id', '0000002', 'multi_region')
        cfg.CONF.set_override('other_region_ids', ['0000001'], 'multi_region')

        intf_segment_dict = self.config_syncer.intf_segment_dict
        segment_nat_dict = self.config_syncer.segment_nat_dict

        conn = self.driver._get_connection()

        parsed_cfg = HTParser(TEST_ACL_RUNNING_CFG)

        invalid_cfg = self.config_syncer.clean_acls(conn,
                                                    intf_segment_dict,
                                                    segment_nat_dict,
                                                    parsed_cfg)

        self.assertEqual([], invalid_cfg)

    def test_clean_nat_pool_overload_basic_running_cfg(self):
        """
        region 1 acls should be ignored
        """
        cfg.CONF.set_override('enable_multi_region', True, 'multi_region')
        cfg.CONF.set_override('region_id', '0000002', 'multi_region')
        cfg.CONF.set_override('other_region_ids', ['0000001'], 'multi_region')

        router_id_dict = self.config_syncer.router_id_dict
        intf_segment_dict = self.config_syncer.intf_segment_dict
        segment_nat_dict = self.config_syncer.segment_nat_dict

        conn = self.driver._get_connection()

        parsed_cfg = HTParser(TEST_ACL_RUNNING_CFG)
        invalid_cfg = self.config_syncer.clean_nat_pool_overload(conn,
                                                     router_id_dict,
                                                     intf_segment_dict,
                                                     segment_nat_dict,
                                                     parsed_cfg)

        self.assertEqual([], invalid_cfg)

    def _test_clean_router_multi_ext_net(self, routers_dict_filename,
                                         running_cfg_filename, hd_id,
                                         expected_invalid_cfg=None):
        if expected_invalid_cfg is None:
            expected_invalid_cfg = []
        path = os.path.dirname(os.path.abspath(__file__)) + '/'
        with open(path + routers_dict_filename) as routers_dict_file:
            r_dict_txt = routers_dict_file.read()
            routers_dict = jsonutils.loads(r_dict_txt)
            with open(path + running_cfg_filename) as running_config_file:
                running_cfg = running_config_file.read()
                hosting_device_info = {'id': hd_id}
                config_syncer = asr1k_cfg_syncer.ConfigSyncer(
                    routers_dict, self.driver, hosting_device_info)
                config_syncer.get_running_config = mock.Mock(
                    return_value=running_cfg)
                invalid_cfg = config_syncer.delete_invalid_cfg()
                self.assertEqual(expected_invalid_cfg, invalid_cfg)

    def test_clean_router_with_two_subnet_gw(self):
        self._test_clean_router_multi_ext_net(
            'json/1/router_dicts_multi_subnets_ext_net.json',
            'json/1/running_cfg_0005_multi_subnets_ext_net.txt',
            '00000000-0000-0000-0000-000000000005')

    def test_clean_ha_backup_router_with_two_subnet_gw(self):
        self._test_clean_router_multi_ext_net(
            'json/1/router_dicts_multi_subnets_ext_net.json',
            'json/1/running_cfg_0004_multi_subnets_ext_net.txt',
            '00000000-0000-0000-0000-000000000004')

    def test_clean_routers_with_two_subnet_gw_and_single_subnet_gw(self):
        self._test_clean_router_multi_ext_net(
                'json/2/router_dicts_multi_subnets_ext_net_and_'
                'single_subnet_ext_net.json',
                'json/2/running_cfg_0005_multi_subnets_ext_net_and_'
                'single_subnet_ext_net.txt',
                '00000000-0000-0000-0000-000000000005')

    def test_clean_ha_backup_routers_with_two_subnet_gw_and_single_subnet_gw(
            self):
        self._test_clean_router_multi_ext_net(
            'json/2/router_dicts_multi_subnets_ext_net_and_'
            'single_subnet_ext_net.json',
            'json/2/running_cfg_0004_multi_subnets_ext_net_and_'
            'single_subnet_ext_net.txt',
            '00000000-0000-0000-0000-000000000004')



# Copyright (c) 2017 Cisco Systems, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock
import testtools

from oslo_config import cfg

from networking_cisco import backwards_compatibility as bc
from networking_cisco.backwards_compatibility import cb_events as events

from networking_cisco.ml2_drivers.nexus import (
    constants as const)
from networking_cisco.services.trunk import nexus_trunk

from neutron.tests.unit.db import test_db_base_plugin_v2
from neutron.tests.unit import testlib_api


PORT_ID = 'fake_port_id'
TRUNK_ID = 'fake_trunk_id'
DNS_NAME = 'test_dns_name'
VM_NAME = 'test_vm_name'
SEGMENTATION_VLAN = 'vlan'
SEGMENTATION_ID1 = 101
SEGMENTATION_ID2 = 102

SUBPORTS = [
    {'segmentation_type': SEGMENTATION_VLAN, 'port_id': PORT_ID,
     'segmentation_id': SEGMENTATION_ID1},
    {'segmentation_type': SEGMENTATION_VLAN, 'port_id': PORT_ID,
     'segmentation_id': SEGMENTATION_ID2}]

TRUNK = {
    'status': bc.constants.PORT_STATUS_ACTIVE,
    'sub_ports': SUBPORTS,
    'name': 'trunk0',
    'admin_state_up': 'true',
    'tenant_id': 'fake_tenant_id',
    'project_id': 'fake_project_id',
    'port_id': PORT_ID,
    'id': TRUNK_ID,
    'description': 'fake trunk port'}

SUBPORT = {
    'status': bc.constants.PORT_STATUS_ACTIVE,
    'port_id': PORT_ID,
    'segmentation_id': SEGMENTATION_ID1}

PORT_BAREMETAL = {
    'status': bc.constants.PORT_STATUS_ACTIVE,
    bc.portbindings.VNIC_TYPE: bc.portbindings.VNIC_BAREMETAL,
    bc.dns.DNSNAME: DNS_NAME,
    bc.portbindings.PROFILE: {"local_link_information": []}}

PORT_VM = {
    'status': bc.constants.PORT_STATUS_ACTIVE,
    bc.portbindings.VNIC_TYPE: bc.portbindings.VNIC_NORMAL,
    bc.portbindings.HOST_ID: VM_NAME,
    bc.portbindings.PROFILE: {}}

PORT_SUBPORT = {
    'status': bc.constants.PORT_STATUS_ACTIVE,
    bc.portbindings.PROFILE: {}}


@testtools.skipIf(bc.NEUTRON_VERSION < bc.NEUTRON_OCATA_VERSION,
                  "Test not applicable prior to stable/ocata.")
class TestNexusTrunkHandler(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
    def setUp(self):
        super(TestNexusTrunkHandler, self).setUp()
        self.handler = nexus_trunk.NexusTrunkHandler()
        self.plugin = bc.get_plugin()
        self.plugin.update_port = mock.Mock()
        self.mock_trunk_get_object = mock.patch.object(
            bc.trunk_objects.Trunk, 'get_object').start()

    def _fake_trunk_payload(self):
        payload = mock.Mock()
        payload.current_trunk.status = bc.constants.PORT_STATUS_DOWN
        payload.current_trunk.to_dict = mock.Mock(return_value=TRUNK)
        payload.original_trunk.status = bc.constants.PORT_STATUS_DOWN
        payload.original_trunk.to_dict = mock.Mock(return_value=TRUNK)
        payload.subports = mock.MagicMock()
        payload.subports[0] = mock.Mock()
        payload.subports[0].segmentation_id = SEGMENTATION_ID1
        payload.subports[0].port_id = PORT_ID
        payload.subports[0].to_dict = mock.Mock(return_value=SUBPORT)
        payload.trunk_id = TRUNK_ID
        return payload

    def _call_test_method(self, test_method, port, event=mock.ANY):
        self.plugin.get_port = mock.Mock(side_effect=[port, PORT_SUBPORT])
        method = getattr(self.handler, test_method)
        method(mock.ANY, event, mock.ANY, self._fake_trunk_payload())

    def _verify_subport_postcommit(self):
        self.assertEqual(1, self.plugin.get_port.call_count)
        self.mock_trunk_get_object.assert_called_once_with(
            mock.ANY, id=TRUNK_ID)

    def _verify_subport_postcommit_unsupported(self):
        self.assertEqual(1, self.plugin.get_port.call_count)
        self.assertFalse(self.plugin.update_port.call_count)
        self.assertFalse(self.mock_trunk_get_object.call_count)

    def _verify_subport_postcommit_unsupported_event(self):
        self.assertEqual(1, self.plugin.get_port.call_count)
        self.assertFalse(self.plugin.update_port.call_count)
        self.mock_trunk_get_object.assert_called_once_with(
            mock.ANY, id=TRUNK_ID)

    def test_trunk_update_postcommit_vm(self):
        self._call_test_method(
            "trunk_update_postcommit", PORT_VM, event=events.PRECOMMIT_DELETE)

        self.plugin.get_port.assert_called_once_with(mock.ANY, PORT_ID)
        self.assertFalse(self.plugin.update_port.call_count)

    def test_trunk_update_postcommit_baremetal_active_state(self):
        self._call_test_method("trunk_update_postcommit", PORT_BAREMETAL)

        self.plugin.get_port.assert_called_once_with(mock.ANY, PORT_ID)
        self.assertFalse(self.plugin.update_port.call_count)

    def test_trunk_update_postcommit_baremetal_down_state(self):
        TRUNK['status'] = bc.constants.PORT_STATUS_DOWN
        self._call_test_method("trunk_update_postcommit", PORT_BAREMETAL)

        self.plugin.get_port.assert_called_once_with(mock.ANY, PORT_ID)
        self.assertEqual(
            len(TRUNK['sub_ports']), self.plugin.update_port.call_count)
        self.plugin.update_port.assert_called_with(mock.ANY, PORT_ID,
            {'port':
             {bc.portbindings.HOST_ID: None,
              'status': bc.constants.PORT_STATUS_DOWN}})

    def test_subport_postcommit_baremetal_after_create(self):
        self._call_test_method(
            "subport_postcommit", PORT_BAREMETAL, event=events.AFTER_CREATE)

        self._verify_subport_postcommit()
        self.plugin.update_port.assert_called_with(mock.ANY, PORT_ID,
            {'port':
             {bc.portbindings.HOST_ID: DNS_NAME,
              'device_owner': bc.trunk_consts.TRUNK_SUBPORT_OWNER}})

    def test_subport_postcommit_baremetal_after_delete(self):
        self._call_test_method(
            "subport_postcommit", PORT_BAREMETAL, event=events.AFTER_DELETE)

        self.plugin.update_port.assert_called_with(mock.ANY, PORT_ID,
            {'port':
             {bc.portbindings.HOST_ID: None,
              'status': bc.constants.PORT_STATUS_DOWN}})

    def test_subport_postcommit_baremetal_unsupported_event(self):
        self._call_test_method(
            "subport_postcommit", PORT_BAREMETAL, event=events.BEFORE_DELETE)

        self._verify_subport_postcommit_unsupported_event()

    def test_subport_postcommit_vm(self):
        self._call_test_method(
            "subport_postcommit", PORT_VM, event=events.AFTER_CREATE)

        self._verify_subport_postcommit_unsupported()

    def test_subport_postcommit_vm_no_hostid(self):
        PORT_VM_NO_HOSTID = PORT_VM.copy()
        del PORT_VM_NO_HOSTID[bc.portbindings.HOST_ID]
        self._call_test_method(
            "subport_postcommit", PORT_VM_NO_HOSTID,
            event=events.AFTER_CREATE)

        self._verify_subport_postcommit_unsupported()

    def test_subport_postcommit_vm_unsupported_event(self):
        self._call_test_method(
            "subport_postcommit", PORT_VM, event=events.BEFORE_DELETE)

        self._verify_subport_postcommit_unsupported()


@testtools.skipIf(bc.NEUTRON_VERSION < bc.NEUTRON_OCATA_VERSION,
                  "Test not applicable prior to stable/ocata.")
class TestNexusTrunkDriver(testlib_api.SqlTestCase):
    def setUp(self):
        super(TestNexusTrunkDriver, self).setUp()

    def test_is_loaded(self):
        driver = nexus_trunk.NexusTrunkDriver.create()
        cfg.CONF.set_override('mechanism_drivers',
                              ["logger", const.CISCO_NEXUS_ML2_MECH_DRIVER_V2],
                              group='ml2')
        self.assertTrue(driver.is_loaded)

        cfg.CONF.set_override('mechanism_drivers',
                              ['logger'],
                              group='ml2')
        self.assertFalse(driver.is_loaded)

        cfg.CONF.set_override('core_plugin', 'some_plugin')
        self.assertFalse(driver.is_loaded)



# Copyright (c) 2017 Cisco Systems, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Replay Test Classes using RESTAPI Driver to test Cisco Nexus platforms.

These Classes are based on the original ssh event driver so same
tests occur with same configuration.  What's different between
the tests is the resulting driver output which is what
the tests in this class presents to its parent class.

You will notice in this file there are test methods which
are skipped by using 'pass'.  This is because these tests
apply to ssh only OR because rerunning the test would be
redundant.
"""

import mock
from oslo_config import cfg

from networking_cisco.ml2_drivers.nexus import (
    constants as const)
from networking_cisco.ml2_drivers.nexus import (
    nexus_db_v2 as nxos_db)
from networking_cisco.ml2_drivers.nexus import (
    nexus_restapi_snippets as snipp)
from networking_cisco.tests.unit.ml2_drivers.nexus import (
    test_cisco_nexus_base as base)
from networking_cisco.tests.unit.ml2_drivers.nexus import (
    test_cisco_nexus_replay)


class TestCiscoNexusRestReplayResults(base.TestCiscoNexusBaseResults):

    """Unit tests driver results for Cisco ML2 Nexus."""

    test_results = {

        'driver_result_unique_init': [
            [(snipp.PATH_IF % 'phys-[eth1/10]'),
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_TRUNKVLAN % ('l1PhysIf', '', '')),
             base.POST],
        ],

        'driver_result_unique_add1': [
            [snipp.PATH_ALL,
             None,
             (snipp.BODY_VLAN_ADD % 267),
             base.POST],
            [(snipp.PATH_IF % 'phys-[eth1/10]'),
             None,
             (snipp.BODY_TRUNKVLAN % ('l1PhysIf', '', '+267')),
             base.POST]
        ],

        'driver_result_unique_add2': [
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_VLAN_ADD % 265),
             base.POST],
            [(snipp.PATH_IF % 'phys-[eth1/10]'),
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_TRUNKVLAN % ('l1PhysIf', '', '+265')),
             base.POST]
        ],

        'driver_result_unique_del1': [
            [(snipp.PATH_IF % 'phys-[eth1/10]'),
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_TRUNKVLAN % ('l1PhysIf', '', '-265')),
             base.POST],
            [(snipp.PATH_VLAN % '265'),
             base.NEXUS_IP_ADDRESS_1,
             '',
             base.DELETE]
        ],

        'driver_result_unique_del2': [
            [(snipp.PATH_IF % 'phys-[eth1/10]'),
             None,
             (snipp.BODY_TRUNKVLAN % ('l1PhysIf', '', '-267')),
             base.POST],
            [(snipp.PATH_VLAN % '267'),
             None,
             '',
             base.DELETE]
        ],

        'driver_result_unique_2vlan_replay': [
            [(snipp.PATH_IF % 'phys-[eth1/10]'),
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_TRUNKVLAN % ('l1PhysIf', '', '+265,267')),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_VLAN_ADD_START % 265) + (
                  snipp.BODY_VLAN_ADD_NEXT % 267) + snipp.BODY_VLAN_ALL_END,
             base.POST]
        ],

        'dupl_vlan_result1_add': [
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_VLAN_ADD % 267),
             base.POST],
            [(snipp.PATH_IF % 'phys-[eth1/10]'),
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_TRUNKVLAN % ('l1PhysIf', '', '+267')),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_VLAN_ADD % 267),
             base.POST],
            [(snipp.PATH_IF % 'phys-[eth1/20]'),
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_TRUNKVLAN % ('l1PhysIf', '', '+267')),
             base.POST]
        ],

        'dupl_vlan_result2_add': [
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_VLAN_ADD % 267),
             base.POST],
            [(snipp.PATH_IF % 'phys-[eth1/10]'),
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_TRUNKVLAN % ('l1PhysIf', '', '+267')),
             base.POST],
            [(snipp.PATH_IF % 'phys-[eth1/20]'),
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_TRUNKVLAN % ('l1PhysIf', '', '+267')),
             base.POST]
        ],

        'dupl_vlan_result2_del': [
            [(snipp.PATH_IF % 'phys-[eth1/10]'),
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_TRUNKVLAN % ('l1PhysIf', '', '-267')),
             base.POST],
            [(snipp.PATH_VLAN % '267'),
             base.NEXUS_IP_ADDRESS_2,
             '',
             base.DELETE],
            [(snipp.PATH_IF % 'phys-[eth1/20]'),
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_TRUNKVLAN % ('l1PhysIf', '', '-267')),
             base.POST]
        ],

        'dupl_vlan_result_replay': [
            [(snipp.PATH_IF % 'phys-[eth1/10]'),
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_TRUNKVLAN % ('l1PhysIf', '', '+267')),
             base.POST],
            [(snipp.PATH_IF % 'phys-[eth1/20]'),
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_TRUNKVLAN % ('l1PhysIf', '', '+267')),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_VLAN_ADD % 267),
             base.POST],
        ],

        'dupl_port_result_replay': [
            [(snipp.PATH_IF % 'phys-[eth1/10]'),
             base.NEXUS_IP_ADDRESS_3,
             (snipp.BODY_TRUNKVLAN % ('l1PhysIf', '', '+267')),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_3,
             (snipp.BODY_VLAN_ADD % 267),
             base.POST],
        ],

        'switch_up_result_add': [
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_DUAL,
             (snipp.BODY_VLAN_ADD % 269),
             base.POST],
            [(snipp.PATH_IF % 'phys-[eth1/3]'),
             base.NEXUS_IP_ADDRESS_DUAL,
             (snipp.BODY_TRUNKVLAN % ('l1PhysIf', '', '+269')),
             base.POST]
        ],

        'switch_up_result_del': [
            [(snipp.PATH_IF % 'phys-[eth1/3]'),
             base.NEXUS_IP_ADDRESS_DUAL,
             (snipp.BODY_TRUNKVLAN % ('l1PhysIf', '', '-269')),
             base.POST],
            [(snipp.PATH_VLAN % '269'),
             base.NEXUS_IP_ADDRESS_DUAL,
             '',
             base.DELETE]
        ],

        'switch_restore_result_add': [
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_DUAL,
             (snipp.BODY_VLAN_ADD % 269),
             base.POST],
            [(snipp.PATH_IF % 'phys-[eth1/3]'),
             base.NEXUS_IP_ADDRESS_DUAL,
             (snipp.BODY_TRUNKVLAN % ('l1PhysIf', '', '+269')),
             base.POST]
        ],

        'switch_restore_result_replay': [
            [(snipp.PATH_IF % 'phys-[eth1/2]'),
             base.NEXUS_IP_ADDRESS_DUAL2,
             (snipp.BODY_TRUNKVLAN % ('l1PhysIf', '', '+269')),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_DUAL2,
             (snipp.BODY_VLAN_ADD % 269),
             base.POST]
        ],

        'switch_restore_result_del': [
            [(snipp.PATH_IF % 'phys-[eth1/3]'),
             base.NEXUS_IP_ADDRESS_DUAL,
             (snipp.BODY_TRUNKVLAN % ('l1PhysIf', '', '-269')),
             base.POST],
            [(snipp.PATH_VLAN % '269'),
             base.NEXUS_IP_ADDRESS_DUAL,
             '',
             base.DELETE],
            [(snipp.PATH_IF % 'phys-[eth1/2]'),
             base.NEXUS_IP_ADDRESS_DUAL2,
             (snipp.BODY_TRUNKVLAN % ('l1PhysIf', '', '-269')),
             base.POST],
            [(snipp.PATH_VLAN % '269'),
             base.NEXUS_IP_ADDRESS_DUAL2,
             '',
             base.DELETE]
        ],

    }


class TestCiscoNexusRestReplay(test_cisco_nexus_replay.TestCiscoNexusReplay):
    """Unit tests for Replay of Cisco ML2 Nexus data."""

    def setUp(self):
        """Sets up mock ncclient, and switch and credentials dictionaries."""

        # Call Grandfather's setUp(); otherwise parent will set driver to
        # 'ncclient' instead of 'restapi'.
        super(test_cisco_nexus_replay.TestCiscoNexusReplay, self).setUp()
        self.results = TestCiscoNexusRestReplayResults()

    def test_replay_unique_ports(self):
        super(TestCiscoNexusRestReplay, self).test_replay_unique_ports()

    def test_replay_duplicate_vlan(self):
        super(TestCiscoNexusRestReplay, self).test_replay_duplicate_vlan()

    def test_replay_duplicate_ports(self):
        super(TestCiscoNexusRestReplay, self).test_replay_duplicate_ports()

    def test_replay_enable_vxlan_feature_failure(self):
        pass

    def test_replay_disable_vxlan_feature_failure(self):
        pass

    def test_replay_create_nve_member_failure(self):
        pass

    def test_replay_delete_nve_member_failure(self):
        pass

    def test_replay_create_vlan_failure(self):
        pass

    def test_replay_delete_vlan_failure(self):
        pass

    def test_replay_create_trunk_failure(self):
        pass

    def test_replay_delete_trunk_failure(self):
        pass

    def test_replay_new_port_success_if_one_switch_up(self):
        (super(TestCiscoNexusRestReplay, self).
            test_replay_new_port_success_if_one_switch_up())

    def test_replay_port_success_if_one_switch_restored(self):
        (super(TestCiscoNexusRestReplay, self).
            test_replay_port_success_if_one_switch_restored())

    def test_replay_create_fails_if_single_switch_down(self):
        (super(TestCiscoNexusRestReplay, self).
            test_replay_create_fails_if_single_switch_down())

    def test_replay_update_fails_if_single_switch_down(self):
        (super(TestCiscoNexusRestReplay, self).
            test_replay_update_fails_if_single_switch_down())

    def test_replay_delete_success_if_switch_down(self):
        (super(TestCiscoNexusRestReplay, self).
            test_replay_delete_success_if_switch_down())

    def test_replay_get_nexus_type_failure_two_switches(self):
        pass

    def test_replay_get_nexus_type_failure(self):
        pass

    def test_replay_create_vlan_failure_during_replay(self):
        pass

    def test_replay_vlan_batch_failure_during_replay(self):
        pass

    def test_replay_no_retry_failure_handling(self):
        pass


class TestCiscoNexusRestBaremetalReplayResults(base.TestCiscoNexusBaseResults):

    """Unit tests driver results for Cisco ML2 Nexus."""

    test_results = {

        'driver_result_unique_init': [
            [(snipp.PATH_IF % 'phys-[eth1/10]'),
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_TRUNKVLAN % ('l1PhysIf', '', '')),
             base.POST],
            [(snipp.PATH_IF % 'phys-[eth1/20]'),
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_TRUNKVLAN % ('l1PhysIf', '', '')),
             base.POST],
        ],

        'driver_result_unique_eth_add1': [
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_VLAN_ADD % 267),
             base.POST],
            [(snipp.PATH_IF % 'phys-[eth1/10]'),
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_NATIVE_TRUNKVLAN % (
                 'l1PhysIf', '', '+267', 'vlan-267')),
             base.POST]
        ],

        'driver_result_unique_eth_add2': [
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_VLAN_ADD % 265),
             base.POST],
            [(snipp.PATH_IF % 'phys-[eth1/20]'),
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_NATIVE_TRUNKVLAN % (
                 'l1PhysIf', '', '+265', 'vlan-265')),
             base.POST]
        ],

        'driver_result_unique_eth_del1': [
            [(snipp.PATH_IF % 'phys-[eth1/20]'),
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_NATIVE_TRUNKVLAN % ('l1PhysIf', '', '-265', '')),
             base.POST],
            [(snipp.PATH_VLAN % '265'),
             base.NEXUS_IP_ADDRESS_1,
             '',
             base.DELETE]
        ],

        'driver_result_unique_eth_del2': [
            [(snipp.PATH_IF % 'phys-[eth1/10]'),
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_NATIVE_TRUNKVLAN % ('l1PhysIf', '', '-267', '')),
             base.POST],
            [(snipp.PATH_VLAN % '267'),
             base.NEXUS_IP_ADDRESS_1,
             '',
             base.DELETE]
        ],

        'driver_result_unique_2if_replay': [
            [(snipp.PATH_IF % 'phys-[eth1/10]'),
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_NATIVE_TRUNKVLAN % (
                 'l1PhysIf', '', '+267', 'vlan-267')),
             base.POST],
            [(snipp.PATH_IF % 'phys-[eth1/20]'),
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_NATIVE_TRUNKVLAN % (
                 'l1PhysIf', '', '+265', 'vlan-265')),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_VLAN_ADD_START % 265) + (
                 snipp.BODY_VLAN_ADD_NEXT % 267) + snipp.BODY_VLAN_ALL_END,
             base.POST]
        ],

        'driver_result_unique_eth_add_vm': [
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_VLAN_ADD % 265),
             base.POST],
            [(snipp.PATH_IF % 'phys-[eth1/10]'),
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_TRUNKVLAN % ('l1PhysIf', '', '+265')),
             base.POST]
        ],

        'driver_result_unique_eth_del_vm': [
            [(snipp.PATH_IF % 'phys-[eth1/10]'),
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_TRUNKVLAN % ('l1PhysIf', '', '-265')),
             base.POST],
            [(snipp.PATH_VLAN % '265'),
             base.NEXUS_IP_ADDRESS_1,
             '',
             base.DELETE]
        ],

        'driver_result_unique_2vlan_replay': [
            [(snipp.PATH_IF % 'phys-[eth1/10]'),
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_NATIVE_TRUNKVLAN % (
                 'l1PhysIf', '', '+267', 'vlan-267')),
             base.POST],
            [(snipp.PATH_IF % 'phys-[eth1/10]'),
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_TRUNKVLAN % ('l1PhysIf', '', '+265,267')),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_VLAN_ADD_START % 265) + (
                 snipp.BODY_VLAN_ADD_NEXT % 267) + snipp.BODY_VLAN_ALL_END,
             base.POST]
        ],
        'driver_result_unique_vPC_2switch_add1': [
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_VLAN_ADD % 267),
             base.POST],
            [(snipp.PATH_IF % 'aggr-[po469]'),
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_NATIVE_TRUNKVLAN % (
                 'pcAggrIf', '', '+267', 'vlan-267')),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_VLAN_ADD % 267),
             base.POST],
            [(snipp.PATH_IF % 'aggr-[po469]'),
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_NATIVE_TRUNKVLAN % (
                 'pcAggrIf', '', '+267', 'vlan-267')),
             base.POST]
        ],

        'driver_result_unique_vPC_2switch_del1': [
            [(snipp.PATH_IF % 'aggr-[po469]'),
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_NATIVE_TRUNKVLAN % (
                 'pcAggrIf', '', '-267', '')),
             base.POST],
            [(snipp.PATH_VLAN % '267'),
             base.NEXUS_IP_ADDRESS_1,
             '',
             base.DELETE],
            [(snipp.PATH_IF % 'aggr-[po469]'),
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_NATIVE_TRUNKVLAN % (
                 'pcAggrIf', '', '-267', '')),
             base.POST],
            [(snipp.PATH_VLAN % '267'),
             base.NEXUS_IP_ADDRESS_2,
             '',
             base.DELETE]
        ],

        'driver_result_unique_vPC_2if_replay': [
            [(snipp.PATH_IF % 'aggr-[po469]'),
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_NATIVE_TRUNKVLAN % (
                 'pcAggrIf', '', '+267', 'vlan-267')),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_VLAN_ADD % 267),
             base.POST],
            [(snipp.PATH_IF % 'aggr-[po469]'),
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_NATIVE_TRUNKVLAN % (
                 'pcAggrIf', '', '+267', 'vlan-267')),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_VLAN_ADD % 267),
             base.POST]
        ],

        'driver_result_unique_vPC470_add1': [
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_VLAN_ADD % 267),
             base.POST],
            [(snipp.PATH_IF % 'aggr-[po470]'),
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_NATIVE_TRUNKVLAN % (
                 'pcAggrIf', '', '+267', 'vlan-267')),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_VLAN_ADD % 267),
             base.POST],
            [(snipp.PATH_IF % 'aggr-[po470]'),
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_NATIVE_TRUNKVLAN % (
                 'pcAggrIf', '', '+267', 'vlan-267')),
             base.POST],
        ],

        'driver_result_unique_vPC470_add2': [
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_VLAN_ADD % 265),
             base.POST],
            [(snipp.PATH_IF % 'aggr-[po470]'),
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_TRUNKVLAN % ('pcAggrIf', '', '+265')),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_VLAN_ADD % 265),
             base.POST],
            [(snipp.PATH_IF % 'aggr-[po470]'),
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_TRUNKVLAN % ('pcAggrIf', '', '+265')),
             base.POST]
        ],

        'driver_result_unique_vPC470_del1': [
            [(snipp.PATH_IF % 'aggr-[po470]'),
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_TRUNKVLAN % ('pcAggrIf', '', '-265')),
             base.POST],
            [(snipp.PATH_VLAN % '265'),
             base.NEXUS_IP_ADDRESS_1,
             '',
             base.DELETE],
            [(snipp.PATH_IF % 'aggr-[po470]'),
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_TRUNKVLAN % ('pcAggrIf', '', '-265')),
             base.POST],
            [(snipp.PATH_VLAN % '265'),
             base.NEXUS_IP_ADDRESS_2,
             '',
             base.DELETE],
        ],

        'driver_result_unique_vPC470_del2': [
            [(snipp.PATH_IF % 'aggr-[po470]'),
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_NATIVE_TRUNKVLAN % ('pcAggrIf', '', '-267', '')),
             base.POST],
            [(snipp.PATH_VLAN % '267'),
             base.NEXUS_IP_ADDRESS_1,
             '',
             base.DELETE],
            [(snipp.PATH_IF % 'aggr-[po470]'),
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_NATIVE_TRUNKVLAN % ('pcAggrIf', '', '-267', '')),
             base.POST],
            [(snipp.PATH_VLAN % '267'),
             base.NEXUS_IP_ADDRESS_2,
             '',
             base.DELETE],
        ],

        'driver_result_unique_vPC470_2vlan_replay': [
            [(snipp.PATH_IF % 'aggr-[po470]'),
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_NATIVE_TRUNKVLAN % (
                 'pcAggrIf', '', '+267', 'vlan-267')),
             base.POST],
            [(snipp.PATH_IF % 'aggr-[po470]'),
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_TRUNKVLAN % ('pcAggrIf', '', '+265,267')),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_VLAN_ADD_START % 265) + (
                  snipp.BODY_VLAN_ADD_NEXT % 267) + snipp.BODY_VLAN_ALL_END,
             base.POST],
            [(snipp.PATH_IF % 'aggr-[po470]'),
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_NATIVE_TRUNKVLAN % (
                 'pcAggrIf', '', '+267', 'vlan-267')),
             base.POST],
            [(snipp.PATH_IF % 'aggr-[po470]'),
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_TRUNKVLAN % ('pcAggrIf', '', '+265,267')),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_VLAN_ADD_START % 265) + (
                  snipp.BODY_VLAN_ADD_NEXT % 267) + snipp.BODY_VLAN_ALL_END,
             base.POST]
        ],

        'driver_result_unique_auto_vPC_2vlan_replay': [
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_ADD_PORT_CH % (1001, 1001, 1001)),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_ADD_PORT_CH_P2 % (1001, 1001)),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_ADD_CH_GRP % (1001, 1001, 'phys-[eth1/10]')),
             base.POST],
            [(snipp.PATH_IF % 'aggr-[po1001]'),
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_TRUNKVLAN % (
                 'pcAggrIf', snipp.BODY_PORT_CH_MODE, '')),
             base.POST],
            [(snipp.PATH_IF % 'aggr-[po1001]'),
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_NATIVE_TRUNKVLAN % (
                 'pcAggrIf', '', '+267', 'vlan-267')),
             base.POST],
            [(snipp.PATH_IF % 'aggr-[po1001]'),
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_TRUNKVLAN % ('pcAggrIf', '', '+265,267')),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_VLAN_ADD_START % 265) + (
                  snipp.BODY_VLAN_ADD_NEXT % 267) + snipp.BODY_VLAN_ALL_END,
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_ADD_PORT_CH % (1001, 1001, 1001)),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_ADD_PORT_CH_P2 % (1001, 1001)),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_ADD_CH_GRP % (1001, 1001, 'phys-[eth1/20]')),
             base.POST],
            [(snipp.PATH_IF % 'aggr-[po1001]'),
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_TRUNKVLAN % (
                 'pcAggrIf', snipp.BODY_PORT_CH_MODE, '')),
             base.POST],
            [(snipp.PATH_IF % 'aggr-[po1001]'),
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_NATIVE_TRUNKVLAN % (
                 'pcAggrIf', '', '+267', 'vlan-267')),
             base.POST],
            [(snipp.PATH_IF % 'aggr-[po1001]'),
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_TRUNKVLAN % ('pcAggrIf', '', '+265,267')),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_VLAN_ADD_START % 265) + (
                  snipp.BODY_VLAN_ADD_NEXT % 267) + snipp.BODY_VLAN_ALL_END,
             base.POST]
        ],

        'driver_result_unique_auto_vPC_vm_add1': [
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_VLAN_ADD % 265),
             base.POST],
            [(snipp.PATH_IF % 'aggr-[po1001]'),
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_TRUNKVLAN % ('pcAggrIf', '', '+265')),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_VLAN_ADD % 265),
             base.POST],
            [(snipp.PATH_IF % 'aggr-[po1001]'),
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_TRUNKVLAN % ('pcAggrIf', '', '+265')),
             base.POST]
        ],

        'driver_result_unique_auto_vPC_vm_del1': [
            [(snipp.PATH_IF % 'aggr-[po1001]'),
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_TRUNKVLAN % ('pcAggrIf', '', '-265')),
             base.POST],
            [(snipp.PATH_VLAN % '265'),
             base.NEXUS_IP_ADDRESS_1,
             '',
             base.DELETE],
            [(snipp.PATH_IF % 'aggr-[po1001]'),
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_TRUNKVLAN % ('pcAggrIf', '', '-265')),
             base.POST],
            [(snipp.PATH_VLAN % '265'),
             base.NEXUS_IP_ADDRESS_2,
             '',
             base.DELETE]
        ],
        'driver_result_unique_auto_vPC_add1': [
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_ADD_PORT_CH % (1001, 1001, 1001)),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_ADD_PORT_CH_P2 % (1001, 1001)),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_ADD_CH_GRP % (1001, 1001, 'phys-[eth1/10]')),
             base.POST],
            [(snipp.PATH_IF % 'aggr-[po1001]'),
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_TRUNKVLAN % (
                 'pcAggrIf', snipp.BODY_PORT_CH_MODE, '')),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_ADD_PORT_CH % (1001, 1001, 1001)),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_ADD_PORT_CH_P2 % (1001, 1001)),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_ADD_CH_GRP % (1001, 1001, 'phys-[eth1/20]')),
             base.POST],
            [(snipp.PATH_IF % 'aggr-[po1001]'),
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_TRUNKVLAN % (
                 'pcAggrIf', snipp.BODY_PORT_CH_MODE, '')),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_VLAN_ADD % 267),
             base.POST],
            [(snipp.PATH_IF % 'aggr-[po1001]'),
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_NATIVE_TRUNKVLAN % (
                 'pcAggrIf', '', '+267', 'vlan-267')),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_VLAN_ADD % 267),
             base.POST],
            [(snipp.PATH_IF % 'aggr-[po1001]'),
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_NATIVE_TRUNKVLAN % (
                 'pcAggrIf', '', '+267', 'vlan-267')),
             base.POST]
        ],
        'driver_result_unique_auto_vPC_del1': [
            [(snipp.PATH_IF % 'aggr-[po1001]'),
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_NATIVE_TRUNKVLAN % ('pcAggrIf', '', '-267', '')),
             base.POST],
            [(snipp.PATH_VLAN % '267'),
             base.NEXUS_IP_ADDRESS_1,
             '',
             base.DELETE],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_DEL_CH_GRP % ('1001', 'phys-[eth1/10]')),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_DEL_PORT_CH % ('1001')),
             base.POST],
            [(snipp.PATH_IF % 'aggr-[po1001]'),
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_NATIVE_TRUNKVLAN % ('pcAggrIf', '', '-267', '')),
             base.POST],
            [(snipp.PATH_VLAN % '267'),
             base.NEXUS_IP_ADDRESS_2,
             '',
             base.DELETE],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_DEL_CH_GRP % ('1001', 'phys-[eth1/20]')),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_DEL_PORT_CH % ('1001')),
             base.POST]
        ],

        'driver_result_unique_auto_vPC_add1_w_user_cfg': [
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_ADD_PORT_CH % (1001, 1001, 1001)),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_ADD_CH_GRP % (1001, 1001, 'phys-[eth1/10]')),
             base.POST],
            [(snipp.PATH_IF % 'aggr-[po1001]'),
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_TRUNKVLAN % (
                 'pcAggrIf', snipp.BODY_PORT_CH_MODE, '')),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_ADD_PORT_CH % (1001, 1001, 1001)),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_ADD_CH_GRP % (1001, 1001, 'phys-[eth1/20]')),
             base.POST],
            [(snipp.PATH_IF % 'aggr-[po1001]'),
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_TRUNKVLAN % (
                 'pcAggrIf', snipp.BODY_PORT_CH_MODE, '')),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_VLAN_ADD % 267),
             base.POST],
            [(snipp.PATH_IF % 'aggr-[po1001]'),
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_NATIVE_TRUNKVLAN % (
                 'pcAggrIf', '', '+267', 'vlan-267')),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_VLAN_ADD % 267),
             base.POST],
            [(snipp.PATH_IF % 'aggr-[po1001]'),
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_NATIVE_TRUNKVLAN % (
                 'pcAggrIf', '', '+267', 'vlan-267')),
             base.POST]
        ],
        'driver_result_unique_auto_vPC_add_usr_cmd_nxapi_cli': [
            [snipp.PATH_USER_CMDS,
             base.NEXUS_IP_ADDRESS_1,
             "int port-channel 1001 ;spanning-tree port type edge trunk "
             ";no lacp suspend-individual",
             base.POST],
            [snipp.PATH_USER_CMDS,
             base.NEXUS_IP_ADDRESS_2,
             "int port-channel 1001 ;spanning-tree port type edge trunk "
             ";no lacp suspend-individual",
             base.POST],
        ],
        'driver_result_unique_auto_vPC_add1_w_user_cfg_replay': [
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_ADD_PORT_CH % (1001, 1001, 1001)),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_ADD_CH_GRP % (1001, 1001, 'phys-[eth1/10]')),
             base.POST],
            [(snipp.PATH_IF % 'aggr-[po1001]'),
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_TRUNKVLAN % (
                 'pcAggrIf', snipp.BODY_PORT_CH_MODE, '')),
             base.POST],
            [(snipp.PATH_IF % 'aggr-[po1001]'),
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_NATIVE_TRUNKVLAN % (
                 'pcAggrIf', '', '+267', 'vlan-267')),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_1,
             (snipp.BODY_VLAN_ADD % 267),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_ADD_PORT_CH % (1001, 1001, 1001)),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_ADD_CH_GRP % (1001, 1001, 'phys-[eth1/20]')),
             base.POST],
            [(snipp.PATH_IF % 'aggr-[po1001]'),
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_TRUNKVLAN % (
                 'pcAggrIf', snipp.BODY_PORT_CH_MODE, '')),
             base.POST],
            [(snipp.PATH_IF % 'aggr-[po1001]'),
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_NATIVE_TRUNKVLAN % (
                 'pcAggrIf', '', '+267', 'vlan-267')),
             base.POST],
            [snipp.PATH_ALL,
             base.NEXUS_IP_ADDRESS_2,
             (snipp.BODY_VLAN_ADD % 267),
             base.POST]
        ],

    }


GET_PORT_CH_RESPONSE = {
    "totalCount": "4",
    "imdata": [
        {
            "pcRsMbrIfs": {
                "attributes": {
                    "parentSKey": "po1",
                    "tSKey": "eth1/11",
                }
            }
        },
        {
            "pcRsMbrIfs": {
                "attributes": {
                    "parentSKey": "po469",
                    "tSKey": "eth1/10",
                }
            }
        },
        {
            "pcRsMbrIfs": {
                "attributes": {
                    "parentSKey": "po2",
                    "tSKey": "eth1/12",
                }
            }
        },
        {
            "pcRsMbrIfs": {
                "attributes": {
                    "parentSKey": "po469",
                    "tSKey": "eth1/20",
                }
            }
        }
    ]
}

GET_INTERFACE_PCHAN_NO_TRUNK_RESPONSE = {
    "totalCount": "1",
    "imdata": [
        {
            "pcAggrIf": {
                "attributes": {
                    "trunkVlans": "1-4094"
                }
            }
        }
    ]
}


class TestCiscoNexusRestBaremetalReplay(
    test_cisco_nexus_replay.TestCiscoNexusBaremetalReplay):

    def get_init_side_effect(
        self, action, ipaddr=None, body=None, headers=None):

        eth_path = 'api/mo/sys/intf/phys-'
        port_chan_path = 'api/mo/sys/intf/aggr-'

        if action == snipp.PATH_GET_NEXUS_TYPE:
            return base.GET_NEXUS_TYPE_RESPONSE
        elif action in snipp.PATH_GET_PC_MEMBERS:
            return GET_PORT_CH_RESPONSE
        elif eth_path in action:
            return base.GET_INTERFACE_RESPONSE
        elif port_chan_path in action:
            return base.GET_INTERFACE_PCHAN_RESPONSE

        return {}

    def get_init_side_effect2(
        self, action, ipaddr=None, body=None, headers=None):

        eth_path = 'api/mo/sys/intf/phys-'
        port_chan_path = 'api/mo/sys/intf/aggr-'

        if action == snipp.PATH_GET_NEXUS_TYPE:
            return base.GET_NEXUS_TYPE_RESPONSE
        elif action in snipp.PATH_GET_PC_MEMBERS:
            return base.GET_NO_PORT_CH_RESPONSE
        elif eth_path in action:
            return base.GET_INTERFACE_RESPONSE
        elif port_chan_path in action:
            return GET_INTERFACE_PCHAN_NO_TRUNK_RESPONSE

        return {}

    def _init_port_channel(self, ch_grp, which=1):

        # this is to prevent interface initialization from occurring
        # which adds unnecessary noise to the results.

        GET_PORT_CH_RESPONSE['imdata'][which]['pcRsMbrIfs'][
            'attributes']['parentSKey'] = ('po' + str(ch_grp))
        data_json = {'rest_get.side_effect':
                    self.get_init_side_effect}
        self.mock_ncclient.configure_mock(**data_json)

    def setUp(self):
        """Sets up mock ncclient, and switch and credentials dictionaries."""
        original_intersect = nxos_db._get_free_vpcids_on_switches

        def new_get_free_vpcids_on_switches(nexus_ips):
            intersect = list(original_intersect(nexus_ips))
            intersect.sort()
            return intersect

        mock.patch.object(nxos_db,
                         '_get_free_vpcids_on_switches',
                         new=new_get_free_vpcids_on_switches).start()
        # Call Grandfather's setUp(); otherwise parent will set driver to
        # 'ncclient' instead of 'restapi'.
        super(test_cisco_nexus_replay.TestCiscoNexusBaremetalReplay,
              self).setUp()
        self.results = TestCiscoNexusRestBaremetalReplayResults()

    def test_replay_unique_ethernet_ports(self):
        (super(TestCiscoNexusRestBaremetalReplay, self).
            test_replay_unique_ethernet_ports())

    def test_replay_unique_ethernet_port_and_vm(self):
        (super(TestCiscoNexusRestBaremetalReplay, self).
            test_replay_unique_ethernet_port_and_vm())

    def test_replay_unique_vPC_ports(self):
        self._init_port_channel(469, 3)
        (super(TestCiscoNexusRestBaremetalReplay, self).
            test_replay_unique_vPC_ports())
        self._init_port_channel(470, 3)

    def test_replay_unique_vPC_ports_and_vm(self):
        self._init_port_channel(470, 3)
        (super(TestCiscoNexusRestBaremetalReplay, self).
            test_replay_unique_vPC_ports_and_vm())

    def test_replay_unique_vPC_ports_chg_vPC_nbr(self):
        self._init_port_channel(469, 3)
        (super(TestCiscoNexusRestBaremetalReplay, self).
            test_replay_unique_vPC_ports_chg_vPC_nbr())
        self._init_port_channel(470, 3)

    def test_replay_unique_vPC_ports_chg_to_enet(self):
        self._init_port_channel(469, 3)
        (super(TestCiscoNexusRestBaremetalReplay, self).
            test_replay_unique_vPC_ports_chg_to_enet())
        self._init_port_channel(470, 3)

    def test_replay_automated_port_channel_w_user_cfg(self):
        """Basic replay of auto-port-channel creation with user config."""

        data_json = {'rest_get.side_effect':
                    self.get_init_side_effect2}
        self.mock_ncclient.configure_mock(**data_json)

        switch_list = ['1.1.1.1', '2.2.2.2']

        for switch_ip in switch_list:
            cfg.CONF.set_override(
                const.VPCPOOL, ('1001-1025'),
                cfg.CONF.ml2_cisco.nexus_switches.get(switch_ip)._group)
        self._cisco_mech_driver._initialize_vpc_alloc_pools()

        self._cfg_vPC_user_commands(
            switch_list, "spanning-tree port type edge trunk;no lacp "
                         "suspend-individual")

        # _init_port_channel is not called so the vpc nbr gets created.

        def replay_complete():
            # Add same together cause this  accounts for the initial add
            # as well as add during replay.
            myresults = (self.results.get_test_results(
                'driver_result_unique_auto_vPC_add_usr_cmd_nxapi_cli') +
                self.results.get_test_results(
                    'driver_result_unique_auto_vPC_add_usr_cmd_nxapi_cli'))
            self._verify_nxapi_results(myresults)

        first_add = {
            'driver_results': self.results.get_test_results(
                'driver_result_unique_auto_vPC_add1_w_user_cfg'),
            'nbr_db_entries': 2}
        second_del = {
            'driver_results': self.results.get_test_results(
                'driver_result_unique_auto_vPC_del1'),
            'nbr_db_entries': 0}

        self._process_replay(
            'test_replay_unique_vPC',
            None,
            [],
            first_add,
            None,
            self.results.get_test_results(
                'driver_result_unique_auto_vPC_add1_w_user_cfg_replay'),
            None,
            second_del,
            replay_complete=replay_complete)

        for switch_ip in switch_list:
            self.assertEqual(
                25, len(nxos_db.get_free_switch_vpc_allocs(switch_ip)))

    def test_replay_automated_vPC_ports_and_vm(self):
        """Provides replay data and result data for unique ports. """

        data_json = {'rest_get.side_effect':
                    self.get_init_side_effect2}
        self.mock_ncclient.configure_mock(**data_json)

        switch_list = ['1.1.1.1', '2.2.2.2']

        for switch_ip in switch_list:
            cfg.CONF.set_override(
                const.VPCPOOL, ('1001-1025'),
                cfg.CONF.ml2_cisco.nexus_switches.get(switch_ip)._group)
        self._cisco_mech_driver._initialize_vpc_alloc_pools()

        # _init_port_channel is not called so the vpc nbr is created

        first_add = {
            'driver_results': self.results.get_test_results(
                'driver_result_unique_auto_vPC_add1'),
            'nbr_db_entries': 2}
        second_add = {
            'driver_results': self.results.get_test_results(
                'driver_result_unique_auto_vPC_vm_add1'),
            'nbr_db_entries': 4}
        first_del = {
            'driver_results': self.results.get_test_results(
                'driver_result_unique_auto_vPC_vm_del1'),
            'nbr_db_entries': 2}
        second_del = {
            'driver_results': self.results.get_test_results(
                'driver_result_unique_auto_vPC_del1'),
            'nbr_db_entries': 0}

        self._process_replay(
            'test_replay_unique_vPC',
            'test_config_vm',
            [],
            first_add,
            second_add,
            self.results.get_test_results(
                'driver_result_unique_auto_vPC_2vlan_replay'),
            first_del,
            second_del)

        for switch_ip in switch_list:
            self.assertEqual(
                25, len(nxos_db.get_free_switch_vpc_allocs(switch_ip)))


#The tests in class below is reproduced this is does not apply to restapis.
#class TestCiscoNexusNonCachedSshReplay(
#    test_cisco_nexus_base.TestCiscoNexusReplayBase):



# Copyright 2015 Cisco Systems, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#


# RPC message type exchange between server and agent.
VM_INFO = 1000
UPDATE_IP_RULE = 1001
UPLINK_NAME = 1002

DID_LEN = 36

# RPC queue name on agent side.
DFA_AGENT_QUEUE = 'dfa_agent'

# RPC queue name on server side.
DFA_SERVER_QUEUE = 'dfa_server_q'
DFA_EXCHANGE = 'dfa'

RESULT_FAIL = 'FAIL'
RESULT_SUCCESS = 'SUCCESS'
CREATE_FAIL = 'CREATE:FAIL'
DELETE_FAIL = 'DELETE:FAIL'
UPDATE_FAIL = 'UPDATE:FAIL'
SUBNET_PENDING = 'SUBNET:PENDING'

DHCP_PORT_CHECK = 3
DHCP_PREFIX = 'dhcp'
LBAAS_PREFIX = 'lbaas'

MAIN_INTERVAL = 5

# Process queues interval
PROCESS_QUE_INTERVAL = 1

# Failure recovery interval
FAIL_REC_INTERVAL = 60

# Heartbeat interval
HB_INTERVAL = 30

# Segmentation ID reuse after 1 hour
SEG_REUSE_TIMEOUT = 1

# Default Orchestrator ID
ORCHESTRATOR_ID = 'Openstack'

# Special return value for an invalid OVS ofport
INVALID_OFPORT = -1
INVALID_VLAN = -1

MIN_VLAN_TAG = 1
MAX_VLAN_TAG = 4094

VM_MSG_TYPE = 50
UPLINK_MSG_TYPE = 51
VM_BULK_SYNC_MSG_TYPE = 52

UPLINK_DET_INTERVAL = 10
ERR_PROC_INTERVAL = 20
# IF 'down' is seen twice continuously
UPLINK_DOWN_THRES = 3

# Timer to check for the presence of flows
FLOW_CHECK_INTERVAL = 60

Q_UPL_PRIO = 1
Q_VM_PRIO = 2

RES_SEGMENT = "SEGMENT"
RES_VLAN = "VLAN"
RES_IN_SUBNET = 'IN_SUB'
RES_OUT_SUBNET = 'OUT_SUB'

uplink_down_reason = "Uplink went down"
uplink_undiscovered_reason = "Uplink not yet discovered"
port_transition_bond_down_reason = \
    "Physical port became port of bond interface, intermittent down"
port_transition_bond_up_reason = \
    "Physical port became port of bond interface, intermittent up"
bridge_not_cfgd_reason = \
    "Integ or Physical Bridge not created by Openstack. Bridge_list: " \
    "%s Integ Br: %s Ext Br: %s"
veth_not_cfgd_reason = \
    "Integ or Physical Patch/Veth Ports not configured by Openstack. " \
    "Ext Peer: %s Integ Peer: %s"
invalid_uplink_ofport_reason = "Uplink OF port not detected on external bridge"
lldp_ofport_not_detect_reason = \
    "lldp veth port not detected on external bridge"
invalid_peer_ofport_reason = \
    "int or phy peer OF Port not detected on Int or Phy Bridge. " \
    "Phy Peer: %s Int Peer: %s"
evb_cfg_fail_reason = "Unable to cfg EVB"



# Copyright 2014 Cisco Systems, Inc.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#


# This class is used instead of the L3AgentNotifyAPI to effectively
# disable notifications from the l3 base class to the l3 agents.
class L3AgentNotifyAPINoOp(object):
    """API for plugin to notify L3 agent but without actions."""
    BASE_RPC_API_VERSION = '1.0'

    def __init__(self, topic=None):
        pass

    def agent_updated(self, context, admin_state_up, host):
        pass

    def router_deleted(self, context, router_id):
        pass

    def routers_updated(self, context, routers, operation=None, data=None):
        pass

    def router_removed_from_agent(self, context, router_id, host):
        pass

    def router_added_to_agent(self, context, routers, host):
        pass

L3AgentNotifyNoOp = L3AgentNotifyAPINoOp()
